@@ -5,12 +5,12 @@ import shutil
 import tempfile
 
 import mock
-from testify import assert_equal
-from testify import run
-from testify import setup
-from testify import teardown
-from testify import TestCase
 
+from testifycompat import assert_equal
+from testifycompat import run
+from testifycompat import setup
+from testifycompat import teardown
+from testifycompat import TestCase
 from tests import mocks
 from tests.assertions import assert_length
 from tron import node
@@ -20,6 +20,7 @@ from tron.api.adapter import ActionRunAdapter
 from tron.api.adapter import JobRunAdapter
 from tron.api.adapter import ReprAdapter
 from tron.api.adapter import RunAdapter
+from tron.core import actiongraph
 from tron.core import actionrun
 from tron.core import job
 
@@ -36,7 +37,7 @@ class MockAdapter(ReprAdapter):
         return 4
 
 
-class ReprAdapterTestCase(TestCase):
+class TestReprAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.original = mock.Mock(one=1, two=2)
@@ -72,7 +73,7 @@ class SampleClassStub(object):
         return "This is false"
 
 
-class ToggleFlagTestCase(TestCase):
+class TestToggleFlag(TestCase):
     @setup
     def setup_stub(self):
         self.stub = SampleClassStub()
@@ -84,14 +85,14 @@ class ToggleFlagTestCase(TestCase):
         assert not self.stub.expects_false()
 
 
-class RunAdapterTestCase(TestCase):
+class TestRunAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.original = mock.Mock()
         self.adapter = RunAdapter(self.original)
 
     def test_get_state(self):
-        assert_equal(self.adapter.get_state(), self.original.state.name)
+        assert_equal(self.adapter.get_state(), self.original.state)
 
     @mock.patch('tron.api.adapter.NodeAdapter', autospec=True)
     def test_get_node(self, mock_node_adapter):
@@ -106,7 +107,7 @@ class RunAdapterTestCase(TestCase):
         assert_equal(self.adapter.get_duration(), '')
 
 
-class ActionRunAdapterTestCase(TestCase):
+class TestActionRunAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.temp_dir = tempfile.mkdtemp()
@@ -128,24 +129,37 @@ class ActionRunAdapterTestCase(TestCase):
         assert_equal(result['command'], self.action_run.rendered_command)
 
 
-class ActionRunGraphAdapterTestCase(TestCase):
+class TestActionRunGraphAdapter(TestCase):
     @setup
     def setup_adapter(self):
+        self.ar1 = mock.MagicMock(action_name="a1")
+        self.ar2 = mock.MagicMock(action_name="a2")
+        self.a1 = mock.MagicMock()
+        self.a2 = mock.MagicMock()
+        self.a1.name = 'a1'
+        self.a2.name = 'a2'
         self.action_runs = mock.create_autospec(
             actionrun.ActionRunCollection,
-            action_graph=mock.MagicMock(),
+            action_graph=actiongraph.ActionGraph(
+                {
+                    'a1': self.a1,
+                    'a2': self.a2
+                },
+                {'a1': set(), 'a2': {'a1'}},
+                {'a1': set(), 'a2': set()},
+            ),
         )
         self.adapter = adapter.ActionRunGraphAdapter(self.action_runs)
-        self.action_run = mock.MagicMock()
-        self.action_runs.__iter__.return_value = [self.action_run]
+        self.action_runs.__iter__.return_value = [self.ar1, self.ar2]
 
     def test_get_repr(self):
         result = self.adapter.get_repr()
-        assert_equal(len(result), 1)
-        assert_equal(self.action_run.id, result[0]['id'])
+        assert len(result) == 2
+        assert self.ar1.id == result[0]['id']
+        assert ['a1'] == result[1]['dependencies']
 
 
-class JobRunAdapterTestCase(TestCase):
+class TestJobRunAdapter(TestCase):
     @setup
     def setup_adapter(self):
         action_runs = mock.MagicMock()
@@ -160,7 +174,7 @@ class JobRunAdapterTestCase(TestCase):
         assert self.adapter.include_action_runs
 
     def test_get_runs(self):
-        with mock.patch('tron.api.adapter.ActionRunAdapter'):
+        with mock.patch('tron.api.adapter.ActionRunAdapter', autospec=True):
             assert_length(self.adapter.get_runs(), 2)
 
     def test_get_runs_without_action_runs(self):
@@ -168,7 +182,7 @@ class JobRunAdapterTestCase(TestCase):
         assert_equal(self.adapter.get_runs(), None)
 
 
-class NodeAdapterTestCase(TestCase):
+class TestNodeAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.node = mock.create_autospec(node.Node)
@@ -180,7 +194,7 @@ class NodeAdapterTestCase(TestCase):
         assert_equal(result['username'], self.node.username)
 
 
-class NodePoolAdapterTestCase(TestCase):
+class TestNodePoolAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.pool = mock.create_autospec(node.NodePool)
@@ -196,7 +210,7 @@ class NodePoolAdapterTestCase(TestCase):
         )
 
 
-class JobIndexAdapterTestCase(TestCase):
+class TestJobIndexAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.job = mock.create_autospec(job.Job)
@@ -230,7 +244,7 @@ class JobIndexAdapterTestCase(TestCase):
         assert_equal(result, [])
 
 
-class SchedulerAdapterTestCase(TestCase):
+class TestSchedulerAdapter(TestCase):
     @setup
     def setup_adapter(self):
         self.scheduler = mock.create_autospec(scheduler.GeneralScheduler)
