@@ -4,8 +4,8 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-from testify.assertions import assert_in
-from testify.assertions import assert_not_reached
+from testifycompat import assert_in
+from testifycompat import assert_not_reached
 
 
 def assert_raises(expected_exception_class, callable_obj, *args, **kwargs):
@@ -27,9 +27,9 @@ def assert_length(sequence, expected, msg=None):
     assert length == expected, msg % locals()
 
 
-def assert_call(turtle, call_idx, *args, **kwargs):
-    """Assert that a function was called on turtle with the correct args."""
-    actual = turtle.calls[call_idx] if turtle.calls else None
+def assert_call(mock, call_idx, *args, **kwargs):
+    """Assert that a function was called on mock with the correct args."""
+    actual = mock.mock_calls[call_idx] if mock.mock_calls else None
     msg = "Call %s expected %s, was %s" % (call_idx, (args, kwargs), actual)
     assert actual == (args, kwargs), msg
 
