@@ -2,12 +2,10 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import mock
-from testify import assert_equal
-from testify import run
-from testify import setup
-from testify import setup_teardown
-from testify import TestCase
 
+from testifycompat import setup
+from testifycompat import setup_teardown
+from testifycompat import TestCase
 from tron.commands import display
 from tron.commands.display import DisplayActionRuns
 from tron.commands.display import DisplayJobRuns
@@ -16,7 +14,7 @@ from tron.core import actionrun
 from tron.core import job
 
 
-class DisplayJobRunsTestCase(TestCase):
+class TestDisplayJobRuns(TestCase):
     @setup
     def setup_data(self):
         self.data = [
@@ -47,10 +45,10 @@ class DisplayJobRunsTestCase(TestCase):
     def test_format(self):
         out = DisplayJobRuns().format(self.data)
         lines = out.split('\n')
-        assert_equal(len(lines), 7)
+        assert len(lines) == 7
 
 
-class DisplayJobsTestCase(TestCase):
+class TestDisplayJobs(TestCase):
     @setup
     def setup_data(self):
         self.data = [
@@ -75,10 +73,10 @@ class DisplayJobsTestCase(TestCase):
 
     def test_format(self):
         lines = self.do_format()
-        assert_equal(len(lines), 5)
+        assert len(lines) == 5
 
 
-class DisplayActionsTestCase(TestCase):
+class TestDisplayActions(TestCase):
     @setup
     def setup_data(self):
         self.data = {
@@ -98,33 +96,54 @@ class DisplayActionsTestCase(TestCase):
                 'sometime',
             'manual':
                 False,
-            'runs':
-                [
-                    dict(
-                        id='something.23.run_other_thing',
-                        state='UNKWN',
-                        start_time='2012-01-23 10:10:10.123456',
-                        end_time='',
-                        duration='',
-                        run_time='sometime',
-                    ),
-                    dict(
-                        id='something.1.run_foo',
-                        state='FAIL',
-                        start_time='2012-01-23 10:10:10.123456',
-                        end_time='2012-01-23 10:40:10.123456',
-                        duration='1234.123456',
-                        run_time='sometime',
-                    ),
-                    dict(
-                        id='something.23.run_other_thing',
-                        state='QUE',
-                        start_time='2012-01-23 10:10:10.123456',
-                        end_time='',
-                        duration='',
-                        run_time='sometime',
-                    ),
-                ],
+            'runs': [
+                dict(
+                    id='something.23.run_other_thing',
+                    state='unknown',
+                    start_time='2012-01-23 10:10:10.123456',
+                    end_time='',
+                    duration='',
+                    run_time='sometime',
+                ),
+                dict(
+                    id='something.1.run_foo',
+                    state='failed',
+                    start_time='2012-01-23 10:10:10.123456',
+                    end_time='2012-01-23 10:40:10.123456',
+                    duration='1234.123456',
+                    run_time='sometime',
+                ),
+                dict(
+                    id='something.23.run_other_thing',
+                    state='queued',
+                    start_time='2012-01-23 10:10:10.123456',
+                    end_time='',
+                    duration='',
+                    run_time='sometime',
+                ),
+                dict(
+                    id='something.42.weird_run',
+                    state='unknown',
+                    start_time=None,
+                    end_time=None,
+                    duration="",
+                    run_time=None,
+                ),
+                dict(
+                    id='something.43.skipped',
+                    state='skipped',
+                    start_time="2019-07-15 18:12:05",
+                    end_time="2019-07-16 01:31:50",
+                    duration="7:19:44.506211",
+                ),
+                dict(
+                    id='something.43.running',
+                    state='running',
+                    start_time="2019-07-15 18:12:05",
+                    end_time=None,
+                    duration="7:19:44.506211",
+                ),
+            ],
         }
         self.details = {
             'id': 'something.1.foo',
@@ -143,25 +162,21 @@ class DisplayActionsTestCase(TestCase):
 
     def test_format(self):
         lines = self.format_lines()
-        assert_equal(len(lines), 13)
+        assert len(lines) == 16, "\n".join(lines)
 
 
-class AddColorForStateTestCase(TestCase):
+class TestAddColorForState(TestCase):
     @setup_teardown
     def enable_color(self):
         with display.Color.enable():
             yield
 
     def test_add_red(self):
-        text = display.add_color_for_state(
-            actionrun.ActionRun.STATE_FAILED.name,
-        )
+        text = display.add_color_for_state(actionrun.ActionRun.FAILED, )
         assert text.startswith(display.Color.colors['red']), text
 
     def test_add_green(self):
-        text = display.add_color_for_state(
-            actionrun.ActionRun.STATE_RUNNING.name,
-        )
+        text = display.add_color_for_state(actionrun.ActionRun.RUNNING, )
         assert text.startswith(display.Color.colors['green']), text
 
     def test_add_blue(self):
@@ -169,7 +184,7 @@ class AddColorForStateTestCase(TestCase):
         assert text.startswith(display.Color.colors['blue']), text
 
 
-class DisplayNodeTestCase(TestCase):
+class TestDisplayNode(TestCase):
 
     node_source = {
         'name': 'name',
@@ -179,29 +194,9 @@ class DisplayNodeTestCase(TestCase):
 
     def test_display_node(self):
         result = display.display_node(self.node_source)
-        assert_equal(result, 'username@hostname')
+        assert result == 'username@hostname'
 
     def test_display_node_pool(self):
         source = {'name': 'name', 'nodes': [self.node_source]}
         result = display.display_node_pool(source)
-        assert_equal(result, 'name (1 node(s))')
-
-
-class DisplaySchedulerTestCase(TestCase):
-    def test_display_scheduler_no_jitter(self):
-        source = {'value': '5 minutes', 'type': 'interval', 'jitter': ''}
-        result = display.display_scheduler(source)
-        assert_equal(result, 'interval 5 minutes')
-
-    def test_display_scheduler_with_jitter(self):
-        source = {
-            'value': '5 minutes',
-            'type': 'interval',
-            'jitter': ' (+/- 2 min)',
-        }
-        result = display.display_scheduler(source)
-        assert_equal(result, 'interval 5 minutes%s' % (source['jitter']))
-
-
-if __name__ == "__main__":
-    run()
+        assert result == 'name (1 node(s))'
