@@ -1,23 +1,21 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import datetime
+from unittest.mock import MagicMock
 
 import mock
+import pytest
 import pytz
-from testify import assert_equal
-from testify import setup
-from testify import TestCase
-from testify.assertions import assert_in
 
+from testifycompat import assert_equal
+from testifycompat import assert_in
+from testifycompat import setup
+from testifycompat import TestCase
 from tests.assertions import assert_call
 from tests.assertions import assert_length
 from tests.assertions import assert_raises
 from tests.testingutils import autospec_method
-from tests.testingutils import Turtle
 from tron import actioncommand
-from tron import event
 from tron import node
+from tron.core import action
 from tron.core import actiongraph
 from tron.core import actionrun
 from tron.core import job
@@ -27,6 +25,12 @@ from tron.serialize import filehandler
 
 def build_mock_job():
     action_graph = mock.create_autospec(actiongraph.ActionGraph)
+    action_graph.action_map = {
+        'foo': mock.Mock(
+            triggered_by=[],
+            trigger_timeout=datetime.timedelta(days=1)
+        )
+    }
     runner = mock.create_autospec(actioncommand.SubprocessActionRunnerFactory)
     return mock.create_autospec(
         job.Job,
@@ -34,12 +38,10 @@ def build_mock_job():
         output_path=mock.Mock(),
         context=mock.Mock(),
         action_runner=runner,
-        service='foo',
-        deploy_group='test',
     )
 
 
-class JobRunTestCase(TestCase):
+class TestJobRun(TestCase):
 
     now = datetime.datetime(2012, 3, 14, 15, 9, 20, tzinfo=None)
     now_with_tz = datetime.datetime(2012, 3, 14, 15, 9, 20, tzinfo=pytz.utc)
@@ -55,16 +57,13 @@ class JobRunTestCase(TestCase):
             7,
             self.run_time,
             mock_node,
-            action_runs=Turtle(
+            action_runs=MagicMock(
                 action_runs_with_cleanup=[],
                 get_startable_action_runs=lambda: [],
             ),
-            service='baz',
-            deploy_group='other',
         )
         autospec_method(self.job_run.watch)
         autospec_method(self.job_run.notify)
-        self.job_run.event = mock.create_autospec(event.EventRecorder)
         self.action_run = mock.create_autospec(
             actionrun.ActionRun,
             is_skipped=False,
@@ -73,7 +72,7 @@ class JobRunTestCase(TestCase):
     def test__init__(self):
         assert_equal(self.job_run.job_name, 'jobname')
         assert_equal(self.job_run.run_time, self.run_time)
-        assert str(self.job_run.output_path).endswith(self.job_run.id)
+        assert str(self.job_run.output_path).endswith(str(self.job_run.run_num))
 
     def test_for_job(self):
         run_num = 6
@@ -91,8 +90,6 @@ class JobRunTestCase(TestCase):
         assert_equal(run.run_num, run_num)
         assert_equal(run.node, mock_node)
         assert not run.manual
-        assert_equal(run.service, self.job.service)
-        assert_equal(run.deploy_group, self.job.deploy_group)
 
     def test_for_job_manual(self):
         run_num = 6
@@ -112,8 +109,6 @@ class JobRunTestCase(TestCase):
         assert_equal(state_data['run_num'], 7)
         assert not state_data['manual']
         assert_equal(state_data['run_time'], self.run_time)
-        assert_equal(state_data['service'], 'baz')
-        assert_equal(state_data['deploy_group'], 'other')
 
     def test_set_action_runs(self):
         self.job_run._action_runs = None
@@ -164,22 +159,11 @@ class JobRunTestCase(TestCase):
     def test_start(self):
         autospec_method(self.job_run._do_start)
         assert self.job_run.start()
-        self.job_run.event.info.assert_called_with('start')
         self.job_run._do_start.assert_called_with()
 
     def test_start_failed(self):
         autospec_method(self.job_run._do_start, return_value=False)
         assert not self.job_run.start()
-        self.job_run.event.info.assert_called_with('start')
-        assert not self.job_run.event.ok.mock_calls
-
-    def test_start_no_startable_action_runs(self):
-        autospec_method(self.job_run._do_start)
-        self.job_run.action_runs.has_startable_action_runs = False
-
-        assert not self.job_run.start()
-        self.job_run.event.info.assert_called_with('start')
-        assert not self.job_run.event.ok.mock_calls
 
     def test_do_start(self):
         startable_runs = [
@@ -192,21 +176,14 @@ class JobRunTestCase(TestCase):
         for startable_run in startable_runs:
             startable_run.start.assert_called_with()
 
-        assert_equal(self.job_run.event.ok.call_count, 1)
-        self.job_run.event.ok.assert_called_with('started')
-
     def test_do_start_all_failed(self):
         autospec_method(self.job_run._start_action_runs, return_value=[None])
         assert not self.job_run._do_start()
-        assert not self.job_run.event.ok.mock_calls
 
     def test_do_start_some_failed(self):
         returns = [True, None]
         autospec_method(self.job_run._start_action_runs, return_value=returns)
-
         assert self.job_run._do_start()
-        assert_equal(self.job_run.event.ok.call_count, 1)
-        self.job_run.event.ok.assert_called_with('started')
 
     def test_do_start_no_runs(self):
         assert not self.job_run._do_start()
@@ -241,6 +218,19 @@ class JobRunTestCase(TestCase):
         started_runs = self.job_run._start_action_runs()
         assert_equal(started_runs, [])
 
+    def test_handler_trigger_ready_still_scheduled(self):
+        autospec_method(self.job_run._start_action_runs)
+        self.job_run.is_scheduled = True
+        self.job_run.handler(self.action_run, actionrun.ActionRun.NOTIFY_TRIGGER_READY)
+        assert not self.job_run._start_action_runs.mock_calls
+
+    def test_handler_trigger_ready_started(self):
+        autospec_method(self.job_run._start_action_runs)
+        self.job_run.is_scheduled = False
+        self.job_run.is_queued = False
+        self.job_run.handler(self.action_run, actionrun.ActionRun.NOTIFY_TRIGGER_READY)
+        assert self.job_run._start_action_runs.call_count == 1
+
     def test_handler_not_end_state_event(self):
         autospec_method(self.job_run.finalize)
         autospec_method(self.job_run._start_action_runs)
@@ -263,8 +253,8 @@ class JobRunTestCase(TestCase):
         startable_run.start.assert_called_with()
         assert not self.job_run.finalize.mock_calls
 
-    def test_handler_is_active(self):
-        self.job_run.action_runs.is_active = True
+    def test_handler_runs_not_done(self):
+        self.job_run.action_runs.is_done = False
         autospec_method(self.job_run._start_action_runs, return_value=[])
         autospec_method(self.job_run.finalize)
         self.job_run.handler(self.action_run, mock.Mock())
@@ -310,21 +300,19 @@ class JobRunTestCase(TestCase):
         assert not self.job_run._start_action_runs.mock_calls
 
     def test_state(self):
-        assert_equal(self.job_run.state, actionrun.ActionRun.STATE_SUCCEEDED)
+        assert_equal(self.job_run.state, actionrun.ActionRun.SUCCEEDED)
 
     def test_state_with_no_action_runs(self):
         self.job_run._action_runs = None
-        assert_equal(self.job_run.state, actionrun.ActionRun.STATE_UNKNOWN)
+        assert_equal(self.job_run.state, actionrun.ActionRun.UNKNOWN)
 
     def test_finalize(self):
         self.job_run.action_runs.is_failed = False
         self.job_run.finalize()
-        self.job_run.event.ok.assert_called_with('succeeded')
         self.job_run.notify.assert_called_with(self.job_run.NOTIFY_DONE)
 
     def test_finalize_failure(self):
         self.job_run.finalize()
-        self.job_run.event.critical.assert_called_with('failed')
         self.job_run.notify.assert_called_with(self.job_run.NOTIFY_DONE)
 
     def test_cleanup(self):
@@ -340,14 +328,14 @@ class JobRunTestCase(TestCase):
 
     def test__getattr__(self):
         assert self.job_run.cancel
-        assert self.job_run.is_queued
+        assert self.job_run.state == 'succeeded'
         assert self.job_run.is_succeeded
 
     def test__getattr__miss(self):
         assert_raises(AttributeError, lambda: self.job_run.bogus)
 
 
-class JobRunFromStateTestCase(TestCase):
+class TestJobRunFromState(TestCase):
     @setup
     def setup_jobrun(self):
         self.action_graph = mock.create_autospec(actiongraph.ActionGraph)
@@ -355,18 +343,16 @@ class JobRunFromStateTestCase(TestCase):
         self.path = ['base', 'path']
         self.output_path = mock.create_autospec(filehandler.OutputPath)
         self.node_pool = mock.create_autospec(node.NodePool)
-        self.action_run_state_data = [
-            {
-                'job_run_id': 'thejobname.22',
-                'action_name': 'blingaction',
-                'state': 'succeeded',
-                'run_time': 'sometime',
-                'start_time': 'sometime',
-                'end_time': 'sometime',
-                'command': 'doit',
-                'node_name': 'thenode',
-            }
-        ]
+        self.action_run_state_data = [{
+            'job_run_id': 'thejobname.22',
+            'action_name': 'blingaction',
+            'state': 'succeeded',
+            'run_time': 'sometime',
+            'start_time': 'sometime',
+            'end_time': 'sometime',
+            'command': 'doit',
+            'node_name': 'thenode',
+        }]
         self.state_data = {
             'job_name': 'thejobname',
             'run_num': 22,
@@ -377,8 +363,6 @@ class JobRunFromStateTestCase(TestCase):
             'runs': self.action_run_state_data,
             'cleanup_run': None,
             'manual': True,
-            'service': 'foo',
-            'deploy_group': 'test',
         }
         self.context = mock.Mock()
 
@@ -397,8 +381,6 @@ class JobRunFromStateTestCase(TestCase):
         assert_equal(run.output_path, self.output_path)
         assert run.context.next
         assert run.action_graph
-        assert_equal(run.service, self.state_data['service'])
-        assert_equal(run.deploy_group, self.state_data['deploy_group'])
 
     def test_from_state_node_no_longer_exists(self):
         run = jobrun.JobRun.from_state(
@@ -414,7 +396,7 @@ class JobRunFromStateTestCase(TestCase):
         assert_equal(run.node, self.node_pool)
 
 
-class MockJobRun(Turtle):
+class MockJobRun(MagicMock):
 
     manual = False
 
@@ -422,37 +404,42 @@ class MockJobRun(Turtle):
 
     @property
     def is_scheduled(self):
-        return self.state == actionrun.ActionRun.STATE_SCHEDULED
+        return self.state == actionrun.ActionRun.SCHEDULED
 
     @property
     def is_queued(self):
-        return self.state == actionrun.ActionRun.STATE_QUEUED
+        return self.state == actionrun.ActionRun.QUEUED
 
     @property
     def is_running(self):
-        return self.state == actionrun.ActionRun.STATE_RUNNING
+        return self.state == actionrun.ActionRun.RUNNING
 
     @property
     def is_starting(self):
-        return self.state == actionrun.ActionRun.STATE_STARTING
+        return self.state == actionrun.ActionRun.STARTING
+
+    @property
+    def is_waiting(self):
+        return self.state == actionrun.ActionRun.WAITING
 
     def __repr__(self):
         return str(self.__dict__)
 
 
-class JobRunCollectionTestCase(TestCase):
+class TestJobRunCollection(TestCase):
     def _mock_run(self, **kwargs):
         return MockJobRun(**kwargs)
 
     @setup
     def setup_runs(self):
-        self.run_collection = jobrun.JobRunCollection(5)
+        self.run_collection = jobrun.JobRunCollection(6)
         self.job_runs = [
-            self._mock_run(state=actionrun.ActionRun.STATE_QUEUED, run_num=4),
-            self._mock_run(state=actionrun.ActionRun.STATE_RUNNING, run_num=3),
+            self._mock_run(state=actionrun.ActionRun.QUEUED, run_num=5),
+            self._mock_run(state=actionrun.ActionRun.WAITING, run_num=4),
+            self._mock_run(state=actionrun.ActionRun.RUNNING, run_num=3),
         ] + [
             self._mock_run(
-                state=actionrun.ActionRun.STATE_SUCCEEDED,
+                state=actionrun.ActionRun.SUCCEEDED,
                 run_num=i,
             ) for i in range(2, 0, -1)
         ]
@@ -460,7 +447,7 @@ class JobRunCollectionTestCase(TestCase):
         self.mock_node = mock.create_autospec(node.Node)
 
     def test__init__(self):
-        assert_equal(self.run_collection.run_limit, 5)
+        assert_equal(self.run_collection.run_limit, 6)
 
     def test_from_config(self):
         job_config = mock.Mock(run_limit=20)
@@ -504,8 +491,8 @@ class JobRunCollectionTestCase(TestCase):
         )
         assert_in(job_run, self.run_collection.runs)
         self.run_collection.remove_old_runs.assert_called_with()
-        assert_equal(job_run.run_num, 5)
-        assert_equal(job_run.job_name, mock_job.get_name.return_value)
+        assert job_run.run_num == 6
+        assert job_run.job_name == mock_job.get_name.return_value
 
     def test_build_new_run_manual(self):
         autospec_method(self.run_collection.remove_old_runs)
@@ -519,7 +506,7 @@ class JobRunCollectionTestCase(TestCase):
         )
         assert_in(job_run, self.run_collection.runs)
         self.run_collection.remove_old_runs.assert_called_with()
-        assert_equal(job_run.run_num, 5)
+        assert job_run.run_num == 6
         assert job_run.manual
 
     def test_cancel_pending(self):
@@ -538,17 +525,17 @@ class JobRunCollectionTestCase(TestCase):
 
     def test_remove_pending(self):
         self.run_collection.remove_pending()
-        assert_length(self.run_collection.runs, 3)
+        assert_length(self.run_collection.runs, 4)
         assert_equal(self.run_collection.runs[0], self.job_runs[1])
         assert_call(self.job_runs[0].cleanup, 0)
 
     def test_get_run_by_state(self):
-        state = actionrun.ActionRun.STATE_SUCCEEDED
+        state = actionrun.ActionRun.SUCCEEDED
         run = self.run_collection.get_run_by_state(state)
-        assert_equal(run, self.job_runs[2])
+        assert_equal(run, self.job_runs[3])
 
     def test_get_run_by_state_no_match(self):
-        state = actionrun.ActionRun.STATE_UNKNOWN
+        state = actionrun.ActionRun.UNKNOWN
         run = self.run_collection.get_run_by_state(state)
         assert_equal(run, None)
 
@@ -571,17 +558,9 @@ class JobRunCollectionTestCase(TestCase):
         assert_equal(run, self.job_runs[-2])
 
     def test_get_run_by_index_invalid_index(self):
-        run = self.run_collection.get_run_by_index(-5)
+        run = self.run_collection.get_run_by_index(-6)
         assert_equal(run, None)
-        run = self.run_collection.get_run_by_index(4)
-        assert_equal(run, None)
-
-    def test_get_run_by_state_short_name(self):
-        run = self.run_collection.get_run_by_state_short_name("RUNN")
-        assert_equal(run, self.job_runs[1])
-
-    def test_get_run_by_state_short_name_no_match(self):
-        run = self.run_collection.get_run_by_state_short_name("FAIL")
+        run = self.run_collection.get_run_by_index(5)
         assert_equal(run, None)
 
     def test_get_newest(self):
@@ -590,7 +569,7 @@ class JobRunCollectionTestCase(TestCase):
 
     def test_get_newest_exclude_manual(self):
         run = self._mock_run(
-            state=actionrun.ActionRun.STATE_RUNNING,
+            state=actionrun.ActionRun.RUNNING,
             run_num=5,
             manual=True,
         )
@@ -606,7 +585,7 @@ class JobRunCollectionTestCase(TestCase):
         run_num = self.run_collection.next_run_num()
         scheduled_run = self._mock_run(
             run_num=run_num,
-            state=actionrun.ActionRun.STATE_SCHEDULED,
+            state=actionrun.ActionRun.SCHEDULED,
         )
         self.run_collection.runs.appendleft(scheduled_run)
         pending = list(self.run_collection.get_pending())
@@ -616,23 +595,23 @@ class JobRunCollectionTestCase(TestCase):
     def test_get_active(self):
         starting_run = self._mock_run(
             run_num=self.run_collection.next_run_num(),
-            state=actionrun.ActionRun.STATE_STARTING,
+            state=actionrun.ActionRun.STARTING,
         )
         self.run_collection.runs.appendleft(starting_run)
         active = list(self.run_collection.get_active())
-        assert_length(active, 2)
-        assert_equal(active, [starting_run, self.job_runs[1]])
+        assert_length(active, 3)
+        assert_equal(active, [starting_run, self.job_runs[1], self.job_runs[2]])
 
     def test_get_active_with_node(self):
         starting_run = self._mock_run(
             run_num=self.run_collection.next_run_num(),
-            state=actionrun.ActionRun.STATE_STARTING,
+            state=actionrun.ActionRun.STARTING,
         )
         starting_run.node = 'differentnode'
         self.run_collection.runs.appendleft(starting_run)
         active = list(self.run_collection.get_active('anode'))
-        assert_length(active, 1)
-        assert_equal(active, [self.job_runs[1]])
+        assert_length(active, 2)
+        assert_equal(active, [self.job_runs[1], self.job_runs[2]])
 
     def test_get_active_none(self):
         active = list(self.run_collection.get_active('bogus'))
@@ -642,7 +621,7 @@ class JobRunCollectionTestCase(TestCase):
         run_num = self.run_collection.next_run_num()
         second_queued = self._mock_run(
             run_num=run_num,
-            state=actionrun.ActionRun.STATE_QUEUED,
+            state=actionrun.ActionRun.QUEUED,
         )
         self.run_collection.runs.appendleft(second_queued)
 
@@ -650,36 +629,12 @@ class JobRunCollectionTestCase(TestCase):
         assert_equal(first_queued, self.job_runs[0])
 
     def test_get_first_queued_no_match(self):
-        self.job_runs[0].state = actionrun.ActionRun.STATE_CANCELLED
+        self.job_runs[0].state = actionrun.ActionRun.CANCELLED
         first_queued = self.run_collection.get_first_queued()
         assert not first_queued
 
-    def test_get_next_to_finish(self):
-        next_run = self.run_collection.get_next_to_finish()
-        assert_equal(next_run, self.job_runs[1])
-
-    def test_get_next_to_finish_by_node(self):
-        self.job_runs[1].node = "seven"
-        scheduled_run = self._mock_run(
-            run_num=self.run_collection.next_run_num(),
-            state=actionrun.ActionRun.STATE_SCHEDULED,
-            node="nine",
-        )
-        self.run_collection.runs.appendleft(scheduled_run)
-
-        next_run = self.run_collection.get_next_to_finish(node="seven")
-        assert_equal(next_run, self.job_runs[1])
-
-    def test_get_next_to_finish_none(self):
-        next_run = self.run_collection.get_next_to_finish(node="seven")
-        assert_equal(next_run, None)
-
-        self.job_runs[1].state = None
-        next_run = self.run_collection.get_next_to_finish()
-        assert_equal(next_run, None)
-
     def test_get_next_run_num(self):
-        assert_equal(self.run_collection.next_run_num(), 5)
+        assert_equal(self.run_collection.next_run_num(), 6)
 
     def test_get_next_run_num_first(self):
         run_collection = jobrun.JobRunCollection(5)
@@ -707,10 +662,10 @@ class JobRunCollectionTestCase(TestCase):
         assert_length(self.run_collection.state_data, len(self.job_runs))
 
     def test_last_success(self):
-        assert_equal(self.run_collection.last_success, self.job_runs[2])
+        assert_equal(self.run_collection.last_success, self.job_runs[3])
 
     def test__str__(self):
-        expected = "JobRunCollection[4(queued), 3(running), 2(succeeded), 1(succeeded)]"
+        expected = "JobRunCollection[5(queued), 4(waiting), 3(running), 2(succeeded), 1(succeeded)]"
         assert_equal(str(self.run_collection), expected)
 
     def test_get_action_runs(self):
@@ -723,3 +678,257 @@ class JobRunCollectionTestCase(TestCase):
         assert_equal(runs, expected)
         for job_run in job_runs:
             job_run.get_action_run.assert_called_with(action_name)
+
+
+class TestJobRunStateTransitions:
+    """Integration test for the state of a job run when actions change state in various ways."""
+
+    @pytest.fixture
+    def mock_event_bus(self):
+        with mock.patch(
+            'tron.core.actionrun.EventBus',
+            autospec=True,
+        ) as mock_event_bus:
+            mock_event_bus.has_event.return_value = True
+            yield mock_event_bus
+
+    @pytest.fixture
+    def job_run(self, tmpdir, mock_event_bus):
+        action_foo = action.Action('foo', 'command', None)
+        action_after_foo = action.Action('after_foo', 'command', None)
+        action_bar = action.Action('bar', 'command', None, triggered_by={'trigger'})
+        action_graph = actiongraph.ActionGraph(
+            action_map={
+                'foo': action_foo,
+                'after_foo': action_after_foo,
+                'bar': action_bar,
+            },
+            required_actions={'foo': set(), 'after_foo': {'foo'}, 'bar': set()},
+            required_triggers={'foo': set(), 'after_foo': set(), 'bar': {'trigger'}},
+        )
+        mock_job = mock.Mock(
+            output_path=filehandler.OutputPath(tmpdir),
+            action_graph=action_graph,
+            action_runner=actioncommand.NoActionRunnerFactory(),
+        )
+        job_run = jobrun.JobRun.for_job(
+            mock_job,
+            run_num=1,
+            run_time=datetime.datetime.now(),
+            node=mock.Mock(),
+            manual=False,
+        )
+        return job_run
+
+    def test_success_path(self, job_run):
+        # Check expected states as actions run normally and succeed.
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        # Run is initially SCHEDULED
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+
+        # After starting, both actions without dependencies start.
+        # Run is STARTING
+        job_run.start()
+        assert foo.is_starting
+        assert bar.is_starting
+        assert job_run.state == actionrun.ActionRun.STARTING
+
+        # Commands start successfully, run is RUNNING.
+        foo.action_command.started()
+        bar.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # Still RUNNING after one of two running actions succeeds
+        bar.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # after_foo starts after its dependency succeeds
+        foo.action_command.exited(0)
+        assert after_foo.is_starting
+        after_foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # SUCCEEDED after all actions succeed
+        after_foo.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.SUCCEEDED
+
+    def test_one_action_fails(self, job_run):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        # bar action fails, job is RUNNING because foo is still running
+        job_run.start()
+        foo.action_command.started()
+        bar.action_command.started()
+        bar.action_command.exited(1)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # after_foo still starts after its dependency succeeds
+        foo.action_command.exited(0)
+        assert after_foo.is_starting
+        after_foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # After running actions finish, run enters FAILED terminal state
+        after_foo.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.FAILED
+
+        # If we skip the failed action, run becomes SUCCEEDED
+        bar.skip()
+        assert job_run.state == actionrun.ActionRun.SUCCEEDED
+
+    def test_one_action_unknown(self, job_run):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+
+        # bar action becomes unknown, job is RUNNING because foo is still running
+        job_run.start()
+        foo.action_command.started()
+        bar.action_command.started()
+        bar.action_command.exited(None)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # after_foo still starts after its dependency succeeds
+        foo.action_command.exited(0)
+        assert after_foo.is_starting
+        after_foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # UNKNOWN after running actions finish
+        after_foo.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.UNKNOWN
+
+    def test_both_unknown_and_failed(self, job_run):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        # bar action becomes unknown, job is RUNNING because foo is still running
+        job_run.start()
+        foo.action_command.started()
+        bar.action_command.started()
+        bar.action_command.exited(None)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # after_foo still starts after its dependency succeeds
+        foo.action_command.exited(0)
+        assert after_foo.is_starting
+        after_foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # A different action fails
+        # Overall run is FAILED
+        after_foo.action_command.exited(1)
+        assert job_run.state == actionrun.ActionRun.FAILED
+
+    def test_required_action_fails(self, job_run):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+
+        # An action (foo) required by another action fails
+        # Run is RUNNING while the other action, bar, is running
+        job_run.start()
+        foo.action_command.started()
+        bar.action_command.started()
+        foo.action_command.exited(1)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # bar action succeeds
+        # after_foo cannot run because its required action failed
+        # So run is FAILED even though after_foo is waiting
+        bar.action_command.exited(0)
+        assert after_foo.is_waiting
+        assert job_run.state == actionrun.ActionRun.FAILED
+
+        # Pretend we reconfigured and after_foo doesn't depend on foo anymore
+        # Run should not be WAITING
+        # Ideally it would still be FAILED, but for now it's UNKNOWN in this case
+        job_run.action_runs.action_graph.required_actions['after_foo'] = {}
+        assert job_run.state == actionrun.ActionRun.UNKNOWN
+
+    def test_required_action_unknown(self, job_run):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        # An action (foo) required by another action becomes unknown
+        # Run is RUNNING while the other action, bar, is running
+        job_run.start()
+        foo.action_command.started()
+        bar.action_command.started()
+        foo.action_command.exited(None)
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # Other action succeeds
+        # after_foo cannot run because its required action is unknown
+        # So run is UNKNOWN even though after_foo is waiting
+        bar.action_command.exited(0)
+        assert after_foo.is_waiting
+        assert job_run.state == actionrun.ActionRun.UNKNOWN
+
+        # Pretend we reconfigured and after_foo doesn't depend on foo anymore
+        # Run should not be waiting
+        job_run.action_runs.action_graph.required_actions['after_foo'] = {}
+        assert job_run.state == actionrun.ActionRun.UNKNOWN
+
+    def test_with_trigger(self, job_run, mock_event_bus):
+        foo = job_run.get_action_run('foo')
+        after_foo = job_run.get_action_run('after_foo')
+        bar = job_run.get_action_run('bar')
+
+        # Start without trigger for bar
+        mock_event_bus.has_event.return_value = False
+        # Job should still start in scheduled state
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+
+        # Only foo is able to start
+        job_run.start()
+        assert foo.is_starting
+        assert bar.is_waiting
+        assert job_run.state == actionrun.ActionRun.STARTING
+
+        foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # after_foo runs normally after foo succeeds
+        foo.action_command.exited(0)
+        assert after_foo.is_starting
+        after_foo.action_command.started()
+        assert job_run.state == actionrun.ActionRun.RUNNING
+
+        # After after_foo succeeds, run is not done
+        # WAITING because bar is still waiting for a trigger
+        after_foo.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.WAITING
+
+        # After trigger is available, job run finishes as normal
+        mock_event_bus.has_event.return_value = True
+        bar.trigger_notify()
+        assert bar.is_starting
+        bar.action_command.started()
+        bar.action_command.exited(0)
+        assert job_run.state == actionrun.ActionRun.SUCCEEDED
+
+    def test_queued(self, job_run):
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+        job_run.queue()
+        assert job_run.state == actionrun.ActionRun.QUEUED
+        job_run.start()
+        assert job_run.state == actionrun.ActionRun.STARTING
+
+    def test_cancel_one(self, job_run):
+        assert job_run.state == actionrun.ActionRun.SCHEDULED
+        job_run.start()
+        assert job_run.state == actionrun.ActionRun.STARTING
+        job_run.get_action_run('after_foo').cancel()
+        assert job_run.state == actionrun.ActionRun.CANCELLED
