@@ -6,11 +6,10 @@ import textwrap
 from subprocess import CalledProcessError
 from textwrap import dedent
 
-from testify import assert_equal
-from testify import assert_gt
-from testify.assertions import assert_in
-from testify.assertions import assert_raises_such_that
+import pytest
 
+from testifycompat import assert_equal
+from testifycompat import assert_gt
 from tests import sandbox
 from tron.core import actionrun
 
@@ -32,7 +31,7 @@ SINGLE_ECHO_CONFIG = BASIC_CONFIG + """
 jobs:
   - name: "echo_job"
     node: local
-    schedule: "interval 1 hour"
+    schedule: "cron 0 * * * *"
     actions:
       - name: "echo_action"
         command: "echo 'Echo!'" """
@@ -46,7 +45,7 @@ ALT_NAMESPACED_ECHO_CONFIG = """
 jobs:
   - name: "echo_job"
     node: local
-    schedule: "interval 1 hour"
+    schedule: "cron 0 * * * *"
     actions:
       - name: "echo_action"
         command: "echo 'Echo!'" """
@@ -57,10 +56,7 @@ TOUCH_CLEANUP_FMT = """
 """
 
 
-def summarize_events(events):
-    return [(event['entity'], event['name']) for event in events]
-
-
+@pytest.mark.skip(reason="We don't have a setup for sandbox tests yet")
 class TrondEndToEndTestCase(sandbox.SandboxTestCase):
     def test_end_to_end_basic(self):
         self.start_with_config(SINGLE_ECHO_CONFIG)
@@ -74,9 +70,6 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
         # reconfigure and confirm results
         second_config = DOUBLE_ECHO_CONFIG + TOUCH_CLEANUP_FMT
         self.sandbox.tronfig(second_config)
-        events = summarize_events(client.events())
-        assert_in(('', 'restoring'), events)
-        assert_in(('MASTER.echo_job.0', 'created'), events)
         assert_equal(client.config('MASTER')['config'], second_config)
 
         # reconfigure, by uploading a third configuration
@@ -92,9 +85,9 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
 
         def wait_on_cleanup():
             return (
-                len(client.job(job_url)['runs']) >= 2
-                and client.action_runs(action_url)['state'] ==
-                actionrun.ActionRun.STATE_SUCCEEDED.name
+                len(client.job(job_url)['runs']) >= 2 and
+                client.action_runs(action_url)['state'] ==
+                actionrun.ActionRun.SUCCEEDED
             )
 
         sandbox.wait_on_sandbox(wait_on_cleanup)
@@ -106,12 +99,12 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
         other_act_run = client.action_runs(another_action_url)
         assert_equal(
             echo_action_run['state'],
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
         assert_equal(echo_action_run['stdout'], ['Echo!'])
         assert_equal(
             other_act_run['state'],
-            actionrun.ActionRun.STATE_FAILED.name,
+            actionrun.ActionRun.FAILED,
         )
 
         now = datetime.datetime.now()
@@ -123,7 +116,7 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
         job_runs_url = client.get_url('%s.1' % echo_job_name)
         assert_equal(
             client.job_runs(job_runs_url)['state'],
-            actionrun.ActionRun.STATE_FAILED.name,
+            actionrun.ActionRun.FAILED,
         )
 
     def test_node_reconfig(self):
@@ -132,7 +125,7 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
             jobs:
                 - name: a_job
                   node: local
-                  schedule: "interval 1s"
+                  schedule: "cron * * * * *"
                   actions:
                     - name: first_action
                       command: "echo something"
@@ -159,7 +152,7 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.job_runs,
             job_url,
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
         self.sandbox.tronfig(second_config)
@@ -173,12 +166,13 @@ class TrondEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_sandbox(wait_on_next_run)
 
 
+@pytest.mark.skip(reason="We don't have a setup for sandbox tests yet")
 class TronCommandsTestCase(sandbox.SandboxTestCase):
     def test_tronview(self):
         self.start_with_config(SINGLE_ECHO_CONFIG)
         expected = """\nServices:\nNo Services\n\n\nJobs:
             Name       State       Scheduler           Last Success
-            MASTER.echo_job   enabled     interval 1:00:00    None
+            MASTER.echo_job   enabled     cron 1:00:00    None
             """
 
         def remove_line_space(s):
@@ -197,19 +191,19 @@ class TronCommandsTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.action_runs,
             cleanup_url,
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
         action_run_url = self.client.get_url('MASTER.echo_job.1.echo_action')
         assert_equal(
             self.client.action_runs(action_run_url)['state'],
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
         job_run_url = self.client.get_url('MASTER.echo_job.1')
         assert_equal(
             self.client.job_runs(job_run_url)['state'],
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
         assert_equal(self.client.job(job_url)['status'], 'enabled')
@@ -228,21 +222,11 @@ class TronCommandsTestCase(sandbox.SandboxTestCase):
         def test_return_code(exc):
             assert_equal(exc.returncode, 1)
 
-        assert_raises_such_that(
-            CalledProcessError,
-            test_return_code,
-            self.sandbox.tronfig,
-            bad_config,
-        )
-
-    def test_tronfig_no_header(self):
-        self.start_with_config(SINGLE_ECHO_CONFIG)
-        namespace = 'second'
-        self.sandbox.tronfig(ALT_NAMESPACED_ECHO_CONFIG, name=namespace)
-        stdout, stderr = self.sandbox.tronfig(name=namespace, no_header=True)
-        assert_equal(stdout.rstrip(), ALT_NAMESPACED_ECHO_CONFIG.rstrip())
+        with pytest.raises(CalledProcessError):
+            test_return_code(self.sandbox.tronfig, bad_config)
 
 
+@pytest.mark.skip(reason="We don't have a setup for sandbox tests yet")
 class JobEndToEndTestCase(sandbox.SandboxTestCase):
     def test_cleanup_on_failure(self):
         config = BASIC_CONFIG + dedent(
@@ -250,7 +234,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         jobs:
           - name: "failjob"
             node: local
-            schedule: "constant"
+            schedule: "daily 04:20"
             actions:
               - name: "failaction"
                 command: "failplz"
@@ -262,14 +246,14 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.action_runs,
             action_run_url,
-            actionrun.ActionRun.STATE_FAILED.name,
+            actionrun.ActionRun.FAILED,
         )
 
         action_run_url = self.client.get_url('MASTER.failjob.1.cleanup')
         sandbox.wait_on_state(
             self.client.action_runs,
             action_run_url,
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
         job_runs = self.client.job(self.client.get_url('MASTER.failjob'),
                                    )['runs']
@@ -281,7 +265,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         jobs:
           - name: "multi_step_job"
             node: local
-            schedule: "constant"
+            schedule: "daily 04:20"
             actions:
               - name: "broken"
                 command: "failingcommand"
@@ -297,22 +281,22 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
             action_run_url,
         )
 
-        waiter(actionrun.ActionRun.STATE_FAILED.name)
+        waiter(actionrun.ActionRun.FAILED)
         self.sandbox.tronctl('skip', 'MASTER.multi_step_job.0.broken')
-        waiter(actionrun.ActionRun.STATE_SKIPPED.name)
+        waiter(actionrun.ActionRun.SKIPPED)
 
         action_run_url = self.client.get_url('MASTER.multi_step_job.0.works')
         sandbox.wait_on_state(
             self.client.action_runs,
             action_run_url,
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
         job_run_url = self.client.get_url('MASTER.multi_step_job.0')
         sandbox.wait_on_state(
             self.client.job_runs,
             job_run_url,
-            actionrun.ActionRun.STATE_SUCCEEDED.name,
+            actionrun.ActionRun.SUCCEEDED,
         )
 
     def test_failure_on_multi_step_job_doesnt_wedge_tron(self):
@@ -322,7 +306,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
                 -   name: "random_failure_job"
                     node: local
                     queueing: true
-                    schedule: "constant"
+                    schedule: "daily 04:20"
                     actions:
                         -   name: "fa"
                             command: "sleep 0.1; failplz"
@@ -340,7 +324,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_sandbox(wait_on_random_failure_job)
 
         job_runs = self.client.job(job_url)['runs']
-        expected = [actionrun.ActionRun.STATE_FAILED.name for _ in range(3)]
+        expected = [actionrun.ActionRun.FAILED for _ in range(3)]
         assert_equal([run['state'] for run in job_runs[-3:]], expected)
 
     def test_cancel_schedules_a_new_run(self):
@@ -368,8 +352,8 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
 
         run_states = [run['state'] for run in self.client.job(job_url)['runs']]
         expected = [
-            actionrun.ActionRun.STATE_SCHEDULED.name,
-            actionrun.ActionRun.STATE_CANCELLED.name,
+            actionrun.ActionRun.SCHEDULED,
+            actionrun.ActionRun.CANCELLED,
         ]
         assert_equal(run_states, expected)
 
@@ -381,7 +365,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
             """
             jobs:
                 -   name: "cancel_overlap"
-                    schedule: "interval 1s"
+                    schedule: "cron * * * * *"
                     queueing: False
                     node: local
                     actions:
@@ -405,7 +389,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.job,
             job_run_url,
-            actionrun.ActionRun.STATE_CANCELLED.name,
+            actionrun.ActionRun.CANCELLED,
         )
 
         action_run_states = [
@@ -413,7 +397,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
             for action_run in self.client.job_runs(job_run_url)['runs']
         ]
         expected = [
-            actionrun.ActionRun.STATE_CANCELLED.name
+            actionrun.ActionRun.CANCELLED
             for _ in range(len(action_run_states))
         ]
         assert_equal(action_run_states, expected)
@@ -424,7 +408,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
            jobs:
               - name: fast_job
                 node: local
-                schedule: constant
+                schedule: daily 04:20
                 actions:
                   - name: single_act
                     command: "sleep 20 && echo good"
@@ -436,21 +420,21 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.action_runs,
             action_run_url,
-            actionrun.ActionRun.STATE_RUNNING.name,
+            actionrun.ActionRun.RUNNING,
         )
 
         self.restart_trond()
 
         assert_equal(
             self.client.job_runs(action_run_url)['state'],
-            actionrun.ActionRun.STATE_UNKNOWN.name,
+            actionrun.ActionRun.UNKNOWN,
         )
 
         next_run_url = self.client.get_url('MASTER.fast_job.-1.single_act')
         sandbox.wait_on_state(
             self.client.action_runs,
             next_run_url,
-            actionrun.ActionRun.STATE_RUNNING.name,
+            actionrun.ActionRun.RUNNING,
         )
 
     def test_trond_restart_job_running_with_dependencies(self):
@@ -459,7 +443,7 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
             jobs:
                 - name: complex_job
                   node: local
-                  schedule: interval 10min
+                  schedule: cron * * * * *
                   actions:
                     - name: first_act
                       command: sleep 20 && echo "I'm waiting"
@@ -479,14 +463,14 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
         sandbox.wait_on_state(
             self.client.action_runs,
             action_run_url,
-            actionrun.ActionRun.STATE_RUNNING.name,
+            actionrun.ActionRun.RUNNING,
         )
 
         self.restart_trond()
 
         assert_equal(
             self.client.job_runs(action_run_url)['state'],
-            actionrun.ActionRun.STATE_UNKNOWN.name,
+            actionrun.ActionRun.UNKNOWN,
         )
 
         for followup_action_run in ('following_act', 'last_act'):
@@ -498,5 +482,5 @@ class JobEndToEndTestCase(sandbox.SandboxTestCase):
             )
             assert_equal(
                 self.client.action_runs(url)['state'],
-                actionrun.ActionRun.STATE_QUEUED.name,
+                actionrun.ActionRun.QUEUED,
             )
