@@ -1,16 +1,17 @@
 from __future__ import absolute_import
 from __future__ import unicode_literals
 
-from testify import assert_equal
-from testify import assert_raises
-from testify import run
-from testify import setup
-from testify import TestCase
+import mock
 
+from testifycompat import assert_equal
+from testifycompat import assert_raises
+from testifycompat import run
+from testifycompat import setup
+from testifycompat import TestCase
 from tron.utils import crontab
 
 
-class ConvertPredefinedTestCase(TestCase):
+class TestConvertPredefined(TestCase):
     def test_convert_predefined_valid(self):
         expected = crontab.PREDEFINED_SCHEDULE['@hourly']
         assert_equal(crontab.convert_predefined('@hourly'), expected)
@@ -23,7 +24,7 @@ class ConvertPredefinedTestCase(TestCase):
         assert_equal(crontab.convert_predefined(line), line)
 
 
-class ParseCrontabTestCase(TestCase):
+class TestParseCrontab(TestCase):
     def test_parse_asterisk(self):
         line = '* * * * *'
         actual = crontab.parse_crontab(line)
@@ -31,8 +32,22 @@ class ParseCrontabTestCase(TestCase):
         assert_equal(actual['hours'], None)
         assert_equal(actual['months'], None)
 
+    @mock.patch('tron.utils.crontab.MinuteFieldParser.parse', autospec=True)
+    @mock.patch('tron.utils.crontab.HourFieldParser.parse', autospec=True)
+    @mock.patch('tron.utils.crontab.MonthdayFieldParser.parse', autospec=True)
+    @mock.patch('tron.utils.crontab.MonthFieldParser.parse', autospec=True)
+    @mock.patch('tron.utils.crontab.WeekdayFieldParser.parse', autospec=True)
+    def test_parse(self, mock_dow, mock_month, mock_monthday, mock_hour, mock_min):
+        line = '* * * * *'
+        actual = crontab.parse_crontab(line)
+        assert_equal(actual['minutes'], mock_min.return_value)
+        assert_equal(actual['hours'], mock_hour.return_value)
+        assert_equal(actual['monthdays'], mock_monthday.return_value)
+        assert_equal(actual['months'], mock_month.return_value)
+        assert_equal(actual['weekdays'], mock_dow.return_value)
 
-class MinuteFieldParserTestCase(TestCase):
+
+class TestMinuteFieldParser(TestCase):
     @setup
     def setup_parser(self):
         self.parser = crontab.MinuteFieldParser()
@@ -66,7 +81,7 @@ class MinuteFieldParserTestCase(TestCase):
         assert_equal(self.parser.parse("1,11-22/2,*/20"), expected)
 
 
-class MonthFieldParserTestCase(TestCase):
+class TestMonthFieldParser(TestCase):
     @setup
     def setup_parser(self):
         self.parser = crontab.MonthFieldParser()
@@ -76,7 +91,7 @@ class MonthFieldParserTestCase(TestCase):
         assert_equal(self.parser.parse("DEC, Jan-Feb, jul, MaR"), expected)
 
 
-class WeekdayFieldParserTestCase(TestCase):
+class TestWeekdayFieldParser(TestCase):
     @setup
     def setup_parser(self):
         self.parser = crontab.WeekdayFieldParser()
@@ -86,7 +101,7 @@ class WeekdayFieldParserTestCase(TestCase):
         assert_equal(self.parser.parse("Sun, 3, FRI, SaT-Sun"), expected)
 
 
-class MonthdayFieldParserTestCase(TestCase):
+class TestMonthdayFieldParser(TestCase):
     @setup
     def setup_parser(self):
         self.parser = crontab.MonthdayFieldParser()
