@@ -4,10 +4,10 @@ from __future__ import unicode_literals
 import datetime
 
 import pytz
-from testify import assert_equal
-from testify import setup
-from testify import TestCase
 
+from testifycompat import assert_equal
+from testifycompat import setup
+from testifycompat import TestCase
 from tests import testingutils
 from tron.utils import timeutils
 from tron.utils.timeutils import DateArithmetic
@@ -15,30 +15,7 @@ from tron.utils.timeutils import duration
 from tron.utils.timeutils import macro_timedelta
 
 
-class ToTimestampTestCase(TestCase):
-    def test_normal_time_with_timezone(self):
-        # 62 minutes after the epoch
-        start_date = pytz.utc.localize(datetime.datetime(1970, 1, 1, 1, 2))
-        assert_equal(timeutils.to_timestamp(start_date), 62 * 60)
-
-    def test_ambiguous_times(self):
-        pacific_tz = pytz.timezone("US/Pacific")
-        before_fall_back = timeutils.to_timestamp(
-            pacific_tz.localize(
-                datetime.datetime(2017, 11, 5, 1, 23),
-                is_dst=True,
-            ),
-        )
-        after_fall_back = timeutils.to_timestamp(
-            pacific_tz.localize(
-                datetime.datetime(2017, 11, 5, 1, 23),
-                is_dst=False,
-            ),
-        )
-        assert_equal(after_fall_back - before_fall_back, 60 * 60)
-
-
-class TimeDeltaTestCase(TestCase):
+class TestTimeDelta(TestCase):
     @setup
     def make_dates(self):
         self.start_nonleap = datetime.datetime(year=2011, month=1, day=1)
@@ -170,7 +147,7 @@ class TimeDeltaTestCase(TestCase):
         )
 
 
-class DurationTestCase(TestCase):
+class TestDuration(TestCase):
     @setup
     def setup_times(self):
         self.earliest = datetime.datetime(2012, 2, 1, 3, 0, 0)
@@ -190,7 +167,7 @@ class DurationTestCase(TestCase):
         assert_equal(duration(None), None)
 
 
-class DeltaTotalSecondsTestCase(TestCase):
+class TestDeltaTotalSeconds(TestCase):
     def test(self):
         expected = 86702.004002999995
         delta = datetime.timedelta(*range(1, 6))
@@ -269,15 +246,15 @@ class DateArithmeticTestCase(testingutils.MockTimeTestCase):
             self._cmp_year('year-%s' % i, dt)
 
     def test_unixtime(self):
-        timestamp = int(timeutils.to_timestamp(self.now))
+        timestamp = int(self.now.timestamp())
         assert_equal(DateArithmetic.parse('unixtime'), timestamp)
 
     def test_unixtime_plus(self):
-        timestamp = int(timeutils.to_timestamp(self.now)) + 100
+        timestamp = int(self.now.timestamp()) + 100
         assert_equal(DateArithmetic.parse('unixtime+100'), timestamp)
 
     def test_unixtime_minus(self):
-        timestamp = int(timeutils.to_timestamp(self.now)) - 99
+        timestamp = int(self.now.timestamp()) - 99
         assert_equal(DateArithmetic.parse('unixtime-99'), timestamp)
 
     def test_daynumber(self):
@@ -307,7 +284,75 @@ class DateArithmeticTestCase(testingutils.MockTimeTestCase):
     def test_bad_date_format(self):
         assert DateArithmetic.parse('~~') is None
 
+    def test_round_day(self):
+        start = datetime.datetime(2019, 3, 30)
+        delta = timeutils.macro_timedelta(start, months=-1)
+        assert (start + delta).day == 28
 
-class DateArithmeticWithTimezoneTestCase(DateArithmeticTestCase):
+
+class DateArithmeticYMDHTest(TestCase):
+    def test_ym_plus(self):
+        def parse(*ym):
+            return DateArithmetic.parse('ym+1', datetime.datetime(*ym))
+
+        assert_equal(parse(2018, 1, 1), '2018-02')
+        assert_equal(parse(2017, 12, 1), '2018-01')
+
+    def test_ym_minus(self):
+        def parse(*ym):
+            return DateArithmetic.parse('ym-1', datetime.datetime(*ym))
+
+        assert_equal(parse(2018, 1, 1), '2017-12')
+        assert_equal(parse(2018, 2, 1), '2018-01')
+
+    def test_ymd_plus(self):
+        def parse(*ymd):
+            return DateArithmetic.parse('ymd+1', datetime.datetime(*ymd))
+
+        assert_equal(parse(2018, 1, 1), '2018-01-02')
+        assert_equal(parse(2018, 1, 31), '2018-02-01')
+
+    def test_ymd_minus(self):
+        def parse(*ymd):
+            return DateArithmetic.parse('ymd-1', datetime.datetime(*ymd))
+
+        assert_equal(parse(2018, 1, 1), '2017-12-31')
+        assert_equal(parse(2018, 1, 2), '2018-01-01')
+
+    def test_ymdh_plus(self):
+        def parse(*ymdh):
+            return DateArithmetic.parse('ymdh+1', datetime.datetime(*ymdh))
+
+        assert_equal(parse(2018, 1, 1, 1), '2018-01-01T02')
+        assert_equal(parse(2018, 1, 31, 23), '2018-02-01T00')
+
+    def test_ymdh_minus(self):
+        def parse(*ymdh):
+            return DateArithmetic.parse('ymdh-1', datetime.datetime(*ymdh))
+
+        assert_equal(parse(2018, 1, 1, 1), '2018-01-01T00')
+        assert_equal(parse(2018, 1, 1, 0), '2017-12-31T23')
+
+    def test_ymdhm_plus(self):
+        def parse(*ymdhm):
+            return DateArithmetic.parse('ymdhm+1', datetime.datetime(*ymdhm))
+
+        assert_equal(parse(2018, 1, 1, 1, 1), '2018-01-01T01:02')
+        assert_equal(parse(2018, 1, 31, 23, 59), '2018-02-01T00:00')
+
+    def test_ymdhm_minus(self):
+        def parse(*ymdhm):
+            return DateArithmetic.parse('ymdhm-1', datetime.datetime(*ymdhm))
+
+        assert_equal(parse(2018, 1, 1, 1, 2), '2018-01-01T01:01')
+        assert_equal(parse(2018, 1, 1, 0, 0), '2017-12-31T23:59')
+
+    def test_ym_minus_round(self):
+        dt = datetime.datetime(2019, 3, 30)
+        s = timeutils.DateArithmetic.parse('ym-1', dt=dt)
+        assert s == '2019-02'
+
+
+class TestDateArithmeticWithTimezone(DateArithmeticTestCase):
 
     now = pytz.timezone("US/Pacific").localize(datetime.datetime(2012, 3, 20))
