@@ -3,14 +3,15 @@ from __future__ import unicode_literals
 
 import datetime
 
-from testify import assert_equal
-from testify import run
-from testify import TestCase
+import pytz
 
+from testifycompat import assert_equal
+from testifycompat import run
+from testifycompat import TestCase
 from tron.utils import trontimespec
 
 
-class GetTimeTestCase(TestCase):
+class TestGetTime(TestCase):
     def test_get_time(self):
         assert_equal(datetime.time(4, 15), trontimespec.get_time("4:15"))
         assert_equal(datetime.time(22, 59), trontimespec.get_time("22:59"))
@@ -20,7 +21,7 @@ class GetTimeTestCase(TestCase):
         assert not trontimespec.get_time("22:61")
 
 
-class TimeSpecificationTestCase(TestCase):
+class TestTimeSpecification(TestCase):
     def _cmp(self, start_time, expected):
         start_time = datetime.datetime(*start_time)
         expected = datetime.datetime(*expected)
@@ -133,6 +134,33 @@ class TimeSpecificationTestCase(TestCase):
         time = time_spec.next_time(start_date, False)
         assert_equal(time, datetime.time(1, 20, 4))
 
+    def test_get_match_dst_spring_forward(self):
+        tz = pytz.timezone('US/Pacific')
+        time_spec = trontimespec.TimeSpecification(
+            hours=[0, 1, 2, 3, 4],
+            minutes=[0],
+            seconds=[0],
+            timezone='US/Pacific',
+        )
+        start = trontimespec.naive_as_timezone(datetime.datetime(2020, 3, 8, 1), tz)
+        # Springing forward, the next hour after 1AM should be 3AM
+        next_time = time_spec.get_match(start)
+        assert next_time.hour == 3
+
+    def test_get_match_dst_fall_back(self):
+        tz = pytz.timezone('US/Pacific')
+        time_spec = trontimespec.TimeSpecification(
+            hours=[0, 1, 2, 3, 4],
+            minutes=[0],
+            seconds=[0],
+            timezone='US/Pacific',
+        )
+        start = trontimespec.naive_as_timezone(datetime.datetime(2020, 11, 1, 1), tz)
+        # Falling back, the next hour after 1AM is 1AM again. But we only run on the first 1AM
+        # Next run time should be 2AM
+        next_time = time_spec.get_match(start)
+        assert next_time.hour == 2
+
 
 if __name__ == "__main__":
     run()
