@@ -8,9 +8,9 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 import functools
-
-import six
-from six.moves.urllib.parse import quote
+import os.path
+import time
+from urllib.parse import quote
 
 from tron import actioncommand
 from tron import scheduler
@@ -43,7 +43,7 @@ class ReprAdapter(object):
         repr_data = {field: getattr(self._obj, field) for field in self.fields}
         translated = {
             field: func()
-            for field, func in six.iteritems(self.translators)
+            for field, func in self.translators.items()
         }
         repr_data.update(translated)
         return repr_data
@@ -77,7 +77,7 @@ class RunAdapter(ReprAdapter):
     """Base class for JobRun and ActionRun adapters."""
 
     def get_state(self):
-        return self._obj.state.name
+        return self._obj.state
 
     def get_node(self):
         return NodeAdapter(self._obj.node).get_repr()
@@ -99,6 +99,7 @@ class ActionRunAdapter(RunAdapter):
         'exit_status',
         'action_name',
         'exit_statuses',
+        'retries_remaining',
     ]
 
     translated_field_names = [
@@ -112,6 +113,10 @@ class ActionRunAdapter(RunAdapter):
         'duration',
         'job_name',
         'run_num',
+        'retries_delay',
+        'in_delay',
+        'triggered_by',
+        'trigger_downstreams',
     ]
 
     def __init__(
@@ -137,21 +142,55 @@ class ActionRunAdapter(RunAdapter):
     @toggle_flag('job_run')
     def get_requirements(self):
         action_name = self._obj.action_name
-        required = self.job_run.action_graph.get_required_actions(action_name)
+        required = self.job_run.action_graph.get_dependencies(action_name)
         return [act.name for act in required]
 
-    def _get_serializer(self):
-        return filehandler.OutputStreamSerializer(self._obj.output_path)
+    def _get_serializer(self, path=None):
+        path = filehandler.OutputPath(path) if path else self._obj.output_path
+        return filehandler.OutputStreamSerializer(path)
+
+    def _get_alternate_output_paths(self):
+        try:
+            namespace, jobname, run_num, action = self._obj.id.split('.')
+        except Exception:
+            return None
+
+        # Check to see if the output might have ended up in any alternate locations.
+        for alt_path in self._obj.STDOUT_PATHS:
+            formatted_alt_path = os.path.join(
+                # This ugliness is getting the "root output directory"
+                self._obj.context.next.next.base.job.output_path.base,
+                alt_path.format(
+                    namespace=namespace,
+                    jobname=jobname,
+                    run_num=run_num,
+                    action=action,
+                ),
+            )
+            if os.path.exists(formatted_alt_path):
+                yield formatted_alt_path
 
     @toggle_flag('include_stdout')
     def get_stdout(self):
         filename = actioncommand.ActionCommand.STDOUT
-        return self._get_serializer().tail(filename, self.max_lines)
+        output = self._get_serializer().tail(filename, self.max_lines)
+        if not output:
+            for alt_path in self._get_alternate_output_paths():
+                output = self._get_serializer(alt_path).tail(filename, self.max_lines)
+                if output:
+                    break
+        return output
 
     @toggle_flag('include_stderr')
     def get_stderr(self):
         filename = actioncommand.ActionCommand.STDERR
-        return self._get_serializer().tail(filename, self.max_lines)
+        output = self._get_serializer().tail(filename, self.max_lines)
+        if not output:
+            for alt_path in self._get_alternate_output_paths():
+                output = self._get_serializer(alt_path).tail(filename, self.max_lines)
+                if output:
+                    break
+        return output
 
     def get_job_name(self):
         return self._obj.job_run_id.rsplit('.', 1)[-2]
@@ -159,20 +198,42 @@ class ActionRunAdapter(RunAdapter):
     def get_run_num(self):
         return self._obj.job_run_id.split('.')[-1]
 
+    def get_retries_delay(self):
+        if self._obj.retries_delay:
+            return str(self._obj.retries_delay)
+
+    def get_in_delay(self):
+        if self._obj.in_delay is not None:
+            return self._obj.in_delay.getTime() - time.time()
+
+    def get_triggered_by(self) -> str:
+        remaining = set(self._obj.remaining_triggers)
+        all_triggers = sorted(self._obj.rendered_triggers)
+        return ', '.join(
+            f"{trig}{' (done)' if trig not in remaining else ''}"
+            for trig in all_triggers
+        )
+
+    def get_trigger_downstreams(self) -> str:
+        triggers_to_emit = self._obj.triggers_to_emit()
+        return ', '.join(sorted(triggers_to_emit))
+
 
 class ActionGraphAdapter(object):
     def __init__(self, action_graph):
         self.action_graph = action_graph
 
     def get_repr(self):
-        def build(action):
+        def build(action_name):
+            action = self.action_graph[action_name]
+            dependencies = self.action_graph.get_dependencies(action_name, include_triggers=True)
             return {
                 'name': action.name,
                 'command': action.command,
-                'dependent': [dep.name for dep in action.dependent_actions],
+                'dependencies': [d.name for d in dependencies],
             }
 
-        return [build(action) for action in self.action_graph.get_actions()]
+        return [build(action) for action in self.action_graph.names(include_triggers=True)]
 
 
 class ActionRunGraphAdapter(object):
@@ -181,21 +242,34 @@ class ActionRunGraphAdapter(object):
 
     def get_repr(self):
         def build(action_run):
-            deps = self.action_runs.action_graph.get_dependent_actions(
-                action_run.action_name,
-            )
+            graph = self.action_runs.action_graph
+            dependencies = graph.get_dependencies(action_run.action_name, include_triggers=True)
             return {
                 'id': action_run.id,
                 'name': action_run.action_name,
                 'command': action_run.rendered_command,
                 'raw_command': action_run.bare_command,
-                'state': action_run.state.name,
+                'state': action_run.state,
                 'start_time': action_run.start_time,
                 'end_time': action_run.end_time,
-                'dependent': [dep.name for dep in deps],
+                'dependencies': [d.name for d in dependencies]
             }
 
-        return [build(action_run) for action_run in self.action_runs]
+        def build_trigger(trigger_name):
+            graph = self.action_runs.action_graph
+            trigger = graph[trigger_name]
+            dependencies = graph.get_dependencies(trigger_name, include_triggers=True)
+            return {
+                'name': trigger.name,
+                'command': trigger.command,
+                'dependencies': [d.name for d in dependencies],
+                'state': 'unknown',
+            }
+
+        return [build(action_run) for action_run in self.action_runs] + [
+            build_trigger(trigger_name)
+            for trigger_name in self.action_runs.action_graph.all_triggers
+        ]
 
 
 class JobRunAdapter(RunAdapter):
@@ -285,7 +359,7 @@ class JobAdapter(ReprAdapter):
         return SchedulerAdapter(self._obj.scheduler).get_repr()
 
     def get_action_names(self):
-        return self._obj.action_graph.names
+        return list(self._obj.action_graph.names())
 
     @toggle_flag('include_node_pool')
     def get_node_pool(self):
