@@ -1,16 +1,10 @@
 """
 Web Controllers for the API.
 """
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import logging
 
-import pkg_resources
-import six
-
-import tron
-from tron.config import schema
+from tron import yaml
+from tron.eventbus import EventBus
 
 log = logging.getLogger(__name__)
 
@@ -23,16 +17,15 @@ class JobCollectionController(object):
     def __init__(self, job_collection):
         self.job_collection = job_collection
 
-    def handle_command(self, command):
-        if command == 'disableall':
-            self.job_collection.disable()
-            return "Disabled all jobs."
+    def handle_command(self, command, old_name=None, new_name=None):
+        if command == 'move':
+            if old_name not in self.job_collection.get_names():
+                return f"Error: {old_name} doesn't exist"
+            if new_name in self.job_collection.get_names():
+                return f"Error: {new_name} exists already"
+            return self.job_collection.move(old_name, new_name)
 
-        if command == 'enableall':
-            self.job_collection.enable()
-            return "Enabled all jobs."
-
-        raise UnknownCommandError("Unknown command %s" % command)
+        raise UnknownCommandError(f"Unknown command {command}. Try running this on an individual job or action run id")
 
 
 class ActionRunController(object):
@@ -46,6 +39,7 @@ class ActionRunController(object):
         'stop',
         'kill',
         'retry',
+        'recover',
     }
 
     def __init__(self, action_run, job_run):
@@ -54,14 +48,19 @@ class ActionRunController(object):
 
     def handle_command(self, command):
         if command not in self.mapped_commands:
-            raise UnknownCommandError("Unknown command %s" % command)
+            raise UnknownCommandError(f"Unknown command {command}. You can only do one of the following to Action runs: {self.mapped_commands}")
 
         if command == 'start' and self.job_run.is_scheduled:
             return (
-                "Action run can not be started if it's job run is still "
+                "Action run cannot be started if its job run is still "
                 "scheduled."
             )
 
+        if command == 'recover' and not self.action_run.is_unknown:
+            return (
+                "Action run cannot be recovered if its state is not unknown."
+            )
+
         if command in ('stop', 'kill'):
             return self.handle_termination(command)
 
@@ -77,8 +76,11 @@ class ActionRunController(object):
 
     def handle_termination(self, command):
         try:
-            getattr(self.action_run, command)()
+            # Extra message is only used for killing mesos action as warning so far.
+            extra_msg = getattr(self.action_run, command)()
             msg = "Attempting to %s %s"
+            if extra_msg is not None:
+                msg = msg + "\n" + extra_msg
             return msg % (command, self.action_run)
         except NotImplementedError as e:
             msg = "Failed to %s: %s"
@@ -118,7 +120,10 @@ class JobRunController(object):
             msg = "Failed to %s, %s in state %s"
             return msg % (command, self.job_run, self.job_run.state)
 
-        raise UnknownCommandError("Unknown command %s" % command)
+        if command == "retry":
+            raise UnknownCommandError("Error: Job runs cannot be retried, only individual actions can. Did you mean 'rerun'?")
+        else:
+            raise UnknownCommandError(f"Unknown command {command}. Only one of the following applies to a Job run: {self.mapped_commands}")
 
 
 class JobController(object):
@@ -138,16 +143,12 @@ class JobController(object):
             runs = self.job_scheduler.manual_start(run_time=run_time)
             return "Created %s" % ",".join(str(run) for run in runs)
 
-        raise UnknownCommandError("Unknown command %s" % command)
-
-
-def format_seq(seq):
-    return "\n# ".join(sorted(seq))
-
-
-def format_mapping(mapping):
-    seq = ("%-30s: %s" % (k, v) for k, v in sorted(six.iteritems(mapping)))
-    return format_seq(seq)
+        if command == "retry":
+            raise UnknownCommandError("Error: A whole Job cannot be retried, only individual actions for a specific job run id can.")
+        elif command in ["stop", "success", "cancel", "fail", "stop"]:
+            raise UnknownCommandError(f"Error: {command} doesn't apply to a whole Job. Please run this on an individual job run id. Hint: try '{self.job_scheduler.get_job()}.-1' for the latest job id")
+        else:
+            raise UnknownCommandError(f"Unknown command {command}. Does it apply to a whole job? Try a specific Job id or individual action")
 
 
 class ConfigController(object):
@@ -155,51 +156,20 @@ class ConfigController(object):
     from the API.
     """
 
-    TEMPLATE_FILE = 'named_config_template.yaml'
-
-    TEMPLATE = pkg_resources.resource_string(
-        tron.__name__,
-        TEMPLATE_FILE,
-    ).decode('utf-8')
-
-    HEADER_END = "{}\n".format(TEMPLATE.split('\n')[-2])
-
     DEFAULT_NAMED_CONFIG = "\njobs:\n"
 
     def __init__(self, mcp):
         self.mcp = mcp
         self.config_manager = mcp.get_config_manager()
 
-    def render_template(self, config_content):
-        container = self.config_manager.load()
-        command_context = container.get_master().command_context or {}
-        context = {
-            'node_names': format_seq(container.get_node_names()),
-            'command_context': format_mapping(command_context),
-        }
-        header_content = self.TEMPLATE % context
-        return header_content + config_content
-
-    def strip_header(self, name, content):
-        if name == schema.MASTER_NAMESPACE:
-            return content
-
-        header_end_index = content.find(self.HEADER_END)
-        if header_end_index > -1:
-            return content[header_end_index + len(self.HEADER_END):]
-        return content
-
     def _get_config_content(self, name):
         if name not in self.config_manager:
             return self.DEFAULT_NAMED_CONFIG
         return self.config_manager.read_raw_config(name)
 
-    def read_config(self, name, add_header=True):
+    def read_config(self, name):
         config_content = self._get_config_content(name)
         config_hash = self.config_manager.get_hash(name)
-
-        if name != schema.MASTER_NAMESPACE and add_header:
-            config_content = self.render_template(config_content)
         return dict(config=config_content, hash=config_hash)
 
     def check_config(self, name, content, config_hash):
@@ -207,25 +177,30 @@ class ConfigController(object):
         if self.config_manager.get_hash(name) != config_hash:
             return "Configuration update will fail: config is stale, try again"
 
-        content = self.strip_header(name, content)
-
         try:
+            content = yaml.load(content)
             self.config_manager.validate_with_fragment(name, content)
         except Exception as e:
             return "Configuration update will fail: %s" % str(e)
 
-        return self.config_manager.check_config(name, content)
-
     def update_config(self, name, content, config_hash):
         """Update a configuration fragment and reload the MCP."""
         if self.config_manager.get_hash(name) != config_hash:
             return "Configuration has changed. Please try again."
-        content = self.strip_header(name, content)
+
+        old_config = self.read_config(name)['config']
         try:
             self.config_manager.write_config(name, content)
             self.mcp.reconfigure()
         except Exception as e:
             log.error("Configuration update failed: %s" % e)
+            log.error("Reconfiguring with the previous good configuration")
+            try:
+                self.config_manager.write_config(name, old_config)
+                self.mcp.reconfigure()
+            except Exception as e:
+                log.error("Could not restore old config: %s" % e)
+                return str(e)
             return str(e)
 
     def delete_config(self, name, content, config_hash):
@@ -245,3 +220,40 @@ class ConfigController(object):
 
     def get_namespaces(self):
         return self.config_manager.get_namespaces()
+
+
+class EventsController:
+    COMMANDS = {'publish', 'discard'}
+
+    def publish(self, event):
+        if not EventBus.instance:
+            return dict(error='EventBus disabled')
+
+        if EventBus.has_event(event):
+            msg = f"event {event} already published"
+            log.warning(msg)
+            return dict(response=msg)
+
+        if not EventBus.publish(event):
+            msg = f'could not publish {event}'
+            log.error(msg)
+            return dict(error=msg)
+
+        return dict(response='OK')
+
+    def discard(self, event):
+        if not EventBus.instance:
+            return dict(error='EventBus disabled')
+
+        if not EventBus.discard(event):
+            msg = f"could not discard {event}"
+            log.error(msg)
+            return dict(error=msg)
+
+        return dict(response='OK')
+
+    def info(self):
+        if not EventBus.instance:
+            return dict(error='EventBus disabled')
+
+        return dict(response=EventBus.instance.event_log)
