@@ -2,14 +2,10 @@
 Web Services Interface used by command-line clients and web frontend to
 view current state, event history and send commands to trond.
 """
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import collections
 import datetime
 import logging
-
-import six
+import traceback
 
 try:
     import simplejson as json
@@ -19,9 +15,12 @@ except ImportError:
 
 from twisted.web import http, resource, static, server
 
-from tron import event
+from tron import __version__
 from tron.api import adapter, controller
 from tron.api import requestargs
+from tron.api.async_resource import AsyncResource
+from tron.metrics import view_all_metrics
+from tron.metrics import meter
 from tron.utils import maybe_decode
 
 log = logging.getLogger(__name__)
@@ -43,18 +42,23 @@ class JSONEncoder(json.JSONEncoder):
         return super(JSONEncoder, self).default(o)
 
 
-def respond(request, response_dict, code=http.OK, headers=None):
+def respond(request, response, code=None, headers=None):
     """Helper to generate a json response"""
+    if code is None:
+        if type(response) is dict and response.get('error'):
+            code = http.INTERNAL_SERVER_ERROR
+        else:
+            code = http.OK
     request.setResponseCode(code)
     request.setHeader(b'content-type', b'application/json; charset=utf-8')
     request.setHeader(b'Access-Control-Allow-Origin', b'*')
-    for key, val in six.iteritems((headers or {})):
+    for key, val in (headers or {}).items():
         request.setHeader(str(key), str(val))
 
     result = json.dumps(
-        response_dict,
+        response,
         cls=JSONEncoder,
-    ) if response_dict else ""
+    ) if response else ""
 
     if type(result) is not bytes:
         result = result.encode('utf8')
@@ -68,19 +72,48 @@ def handle_command(request, api_controller, obj, **kwargs):
     log.info("Handling '%s' request on %s", command, obj)
     try:
         response = api_controller.handle_command(command, **kwargs)
-        return respond(request, {'result': response})
-    except controller.UnknownCommandError as e:
-        log.warning("Unknown command %s for %s", command, obj)
-        return respond(request, {'error': str(e)}, code=http.NOT_IMPLEMENTED)
+        return respond(request=request, response={'result': response})
+    except controller.UnknownCommandError:
+        error_msg = f"Unknown command '{command}' for '{obj}'"
+        log.warning(error_msg)
+        return respond(
+            request=request, response={'error': error_msg}, code=http.NOT_IMPLEMENTED
+        )
+    except Exception as e:
+        log.exception('%r while executing command %s for %s', e, command, obj)
+        trace = traceback.format_exc()
+        return respond(request=request, response={'error': trace})
+
+
+class ErrorResource(resource.Resource):
+    """ Equivalent to resource.NoResource, except error message is returned
+    as JSON, not HTML """
+
+    def __init__(self, error='No Such Resource', code=http.NOT_FOUND):
+        resource.Resource.__init__(self)
+        self.code = code
+        self.error = error
+
+    @AsyncResource.bounded
+    def render_GET(self, request):
+        return respond(request=request, response={'error': self.error}, code=self.code)
+
+    @AsyncResource.exclusive
+    def render_POST(self, request):
+        return respond(request=request, response={'error': self.error}, code=self.code)
+
+    def getChild(self, chnam, request):
+        """ Overrided getChild to ensure a NoResource is not returned """
+        return self
 
 
 def resource_from_collection(collection, name, child_resource):
     """Return a child resource from a collection by name.  If no item is found,
-    return NoResource.
+    return ErrorResource.
     """
     item = collection.get_by_name(name)
     if item is None:
-        return resource.NoResource("Cannot find child %s" % name)
+        return ErrorResource("Cannot find child '%s'" % name)
     return child_resource(item)
 
 
@@ -94,6 +127,7 @@ class ActionRunResource(resource.Resource):
         self.job_run = job_run
         self.controller = controller.ActionRunController(action_run, job_run)
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         run_adapter = adapter.ActionRunAdapter(
             self.action_run,
@@ -102,8 +136,9 @@ class ActionRunResource(resource.Resource):
             include_stdout=requestargs.get_bool(request, 'include_stdout'),
             include_stderr=requestargs.get_bool(request, 'include_stderr'),
         )
-        return respond(request, run_adapter.get_repr())
+        return respond(request=request, response=run_adapter.get_repr())
 
+    @AsyncResource.exclusive
     def render_POST(self, request):
         return handle_command(request, self.controller, self.action_run)
 
@@ -120,15 +155,16 @@ class JobRunResource(resource.Resource):
             return self
 
         action_name = maybe_decode(action_name)
-        if action_name == '_events':
-            return EventResource(self.job_run.id)
         if action_name in self.job_run.action_runs:
             action_run = self.job_run.action_runs[action_name]
             return ActionRunResource(action_run, self.job_run)
 
-        msg = "Cannot find action %s for %s"
-        return resource.NoResource(msg % (action_name, self.job_run))
+        return ErrorResource(
+            f"Cannot find action '{action_name}' for "
+            f"'{self.job_run}'"
+        )
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         include_runs = requestargs.get_bool(request, 'include_action_runs')
         include_graph = requestargs.get_bool(request, 'include_action_graph')
@@ -137,8 +173,9 @@ class JobRunResource(resource.Resource):
             include_action_runs=include_runs,
             include_action_graph=include_graph,
         )
-        return respond(request, run_adapter.get_repr())
+        return respond(request=request, response=run_adapter.get_repr())
 
+    @AsyncResource.exclusive
     def render_POST(self, request):
         return handle_command(request, self.controller, self.job_run)
 
@@ -161,27 +198,24 @@ class JobResource(resource.Resource):
             return job_runs.get_run_by_num(int(run_id))
         if is_negative_int(run_id):
             return job_runs.get_run_by_index(int(run_id))
-        return job_runs.get_run_by_state_short_name(run_id)
 
     def getChild(self, run_id, _):
         if not run_id:
             return self
 
         run_id = maybe_decode(run_id)
-        if run_id == '_events':
-            return EventResource(self.job_scheduler.get_name())
-
         run = self.get_run_from_identifier(run_id)
         if run:
             return JobRunResource(run, self.job_scheduler)
 
         job = self.job_scheduler.get_job()
-        if run_id in job.action_graph.names:
+        if run_id in job.action_graph.names():
             action_runs = job.runs.get_action_runs(run_id)
             return ActionRunHistoryResource(action_runs)
-        msg = "Cannot find job run %s for %s"
-        return resource.NoResource(msg % (run_id, job))
 
+        return ErrorResource(f"Cannot find job run '{run_id}' for '{job}'")
+
+    @AsyncResource.bounded
     def render_GET(self, request):
         include_action_runs = requestargs.get_bool(
             request,
@@ -196,8 +230,9 @@ class JobResource(resource.Resource):
             include_action_graph=include_graph,
             num_runs=num_runs,
         )
-        return respond(request, job_adapter.get_repr())
+        return respond(request=request, response=job_adapter.get_repr())
 
+    @AsyncResource.exclusive
     def render_POST(self, request):
         run_time = requestargs.get_datetime(request, 'run_time')
         return handle_command(
@@ -216,10 +251,11 @@ class ActionRunHistoryResource(resource.Resource):
         resource.Resource.__init__(self)
         self.action_runs = action_runs
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         return respond(
-            request,
-            adapter.adapt_many(adapter.ActionRunAdapter, self.action_runs),
+            request=request,
+            response=adapter.adapt_many(adapter.ActionRunAdapter, self.action_runs),
         )
 
 
@@ -260,6 +296,7 @@ class JobCollectionResource(resource.Resource):
         )
         return {job['name']: job['actions'] for job in jobs}
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         include_job_runs = requestargs.get_bool(
             request,
@@ -281,7 +318,7 @@ class JobCollectionResource(resource.Resource):
             'include_node_pool',
             default=True,
         )
-        output = dict(
+        response = dict(
             jobs=self.get_data(
                 include_job_runs,
                 include_action_runs,
@@ -289,10 +326,19 @@ class JobCollectionResource(resource.Resource):
                 include_node_pool,
             ),
         )
-        return respond(request, output)
+        return respond(request=request, response=response)
 
+    @AsyncResource.exclusive
     def render_POST(self, request):
-        return handle_command(request, self.controller, self.job_collection)
+        old_name = requestargs.get_string(request, 'old_name')
+        new_name = requestargs.get_string(request, 'new_name')
+        return handle_command(
+            request=request,
+            api_controller=self.controller,
+            obj=self.job_collection,
+            old_name=old_name,
+            new_name=new_name,
+        )
 
 
 class ConfigResource(resource.Resource):
@@ -307,21 +353,19 @@ class ConfigResource(resource.Resource):
     def get_config_index(self):
         return self.controller.get_namespaces()
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         config_name = requestargs.get_string(request, 'name')
-        no_header = requestargs.get_bool(request, 'no_header')
         if not config_name:
             return respond(
-                request,
-                {'error': "'name' for config is required."},
+                request=request,
+                response={'error': "'name' for config is required."},
                 code=http.BAD_REQUEST,
             )
-        response = self.controller.read_config(
-            config_name,
-            add_header=not no_header,
-        )
-        return respond(request, response)
+        response = self.controller.read_config(config_name)
+        return respond(request=request, response=response)
 
+    @AsyncResource.exclusive
     def render_POST(self, request):
         config_content = requestargs.get_string(request, 'config')
         name = requestargs.get_string(request, 'name')
@@ -330,8 +374,8 @@ class ConfigResource(resource.Resource):
 
         if not name:
             return respond(
-                request,
-                {'error': "'name' for config is required."},
+                request=request,
+                response={'error': "'name' for config is required."},
                 code=http.BAD_REQUEST,
             )
 
@@ -352,7 +396,7 @@ class ConfigResource(resource.Resource):
 
         if error:
             response['error'] = error
-        return respond(request, response)
+        return respond(request=request, response=response)
 
 
 class StatusResource(resource.Resource):
@@ -363,25 +407,54 @@ class StatusResource(resource.Resource):
         self._master_control = master_control
         resource.Resource.__init__(self)
 
+    @AsyncResource.bounded
     def render_GET(self, request):
-        return respond(request, {'status': "I'm alive."})
+        return respond(
+            request=request, response={
+                'status': "I'm alive.",
+                'version': __version__,
+                'boot_time': int(self._master_control.boot_time),
+            }
+        )
 
 
-class EventResource(resource.Resource):
+class MetricsResource(resource.Resource):
 
     isLeaf = True
 
-    def __init__(self, entity_name):
+    def __init__(self):
         resource.Resource.__init__(self)
-        self.entity_name = entity_name
 
+    @AsyncResource.exclusive
     def render_GET(self, request):
-        recorder = event.get_recorder(self.entity_name)
-        response_data = adapter.adapt_many(
-            adapter.EventAdapter,
-            recorder.list(),
-        )
-        return respond(request, dict(data=response_data))
+        return respond(request=request, response=view_all_metrics())
+
+
+class EventsResource(resource.Resource):
+    isLeaf = True
+
+    def __init__(self):
+        super().__init__()
+        self.controller = controller.EventsController()
+
+    @AsyncResource.exclusive
+    def render_GET(self, request):
+        response = self.controller.info()
+        return respond(request=request, response=response)
+
+    @AsyncResource.bounded
+    def render_POST(self, request):
+        command = requestargs.get_string(request, 'command')
+        if command not in self.controller.COMMANDS:
+            return respond(
+                request=request,
+                response=dict(error=f'Unknown command: {command}'),
+                code=http.BAD_REQUEST,
+            )
+        event = requestargs.get_string(request, 'event')
+        fn = getattr(self.controller, command)
+        response = fn(event)
+        return respond(request=request, response=response)
 
 
 class ApiRootResource(resource.Resource):
@@ -397,16 +470,18 @@ class ApiRootResource(resource.Resource):
 
         self.putChild(b'config', ConfigResource(mcp))
         self.putChild(b'status', StatusResource(mcp))
-        self.putChild(b'events', EventResource(''))
+        self.putChild(b'events', EventsResource())
+        self.putChild(b'metrics', MetricsResource())
         self.putChild(b'', self)
 
+    @AsyncResource.bounded
     def render_GET(self, request):
         """Return an index of urls for resources."""
         response = {
             'jobs': self.children[b'jobs'].get_job_index(),
             'namespaces': self.children[b'config'].get_config_index(),
         }
-        return respond(request, response)
+        return respond(request=request, response=response)
 
 
 class RootResource(resource.Resource):
@@ -419,7 +494,7 @@ class RootResource(resource.Resource):
         self.putChild(b'', self)
 
     def render_GET(self, request):
-        request.redirect(request.prePathURL() + b'web')
+        request.redirect(b'/web')
         request.finish()
         return server.NOT_DONE_YET
 
@@ -451,5 +526,16 @@ class TronSite(server.Site):
         server.Site.startFactory(self)
         self.logFile = LogAdapter(self.access_log)
 
+    def log(self, request):
+        super().log(request)
+        if 200 <= request.code < 300:
+            meter('tron.site.2xx')
+        if 300 <= request.code < 400:
+            meter('tron.site.3xx')
+        if 400 <= request.code < 500:
+            meter('tron.site.4xx')
+        if 500 <= request.code < 600:
+            meter('tron.site.5xx')
+
     def __repr__(self):
         return '%s(%s)' % (self.__class__.__name__, self.resource)
