@@ -6,6 +6,7 @@ from __future__ import unicode_literals
 from __future__ import with_statement
 
 import argparse
+import difflib
 import logging
 import os
 import sys
@@ -34,6 +35,7 @@ DEFAULT_PORT = 8089
 DEFAULT_CONFIG = {
     'server': "http://%s:%d" % (DEFAULT_HOST, DEFAULT_PORT),
     'display_color': False,
+    'cluster_name': 'Unnamed Cluster',
 }
 
 TAB_COMPLETE_FILE = '/var/cache/tron_tab_completions'
@@ -47,7 +49,7 @@ def get_default_server():
 
 def filter_jobs_actions_runs(prefix, inputs):
     dots = prefix.count('.')
-    if prefix is "":
+    if prefix == "":
         # If the user hasn't begun to type anything, we need to get them started with all jobs
         return [i for i in inputs if i.count('.') == 1]
     elif dots == 0:
@@ -80,10 +82,13 @@ def tron_jobs_completer(prefix, **kwargs):
             inputs=[job.strip('\n\r') for job in jobs],
         )
     else:
-        default_client = Client(get_default_server())
+        if 'client' not in kwargs:
+            client = Client(get_default_server())
+        else:
+            client = kwargs['client']
         return filter_jobs_actions_runs(
             prefix=prefix,
-            inputs=[job['name'] for job in default_client.jobs()],
+            inputs=[job['name'] for job in client.jobs()],
         )
 
 
@@ -111,6 +116,11 @@ def build_option_parser(usage=None, epilog=None):
         default=None,
         help="Url including scheme, host and port, Default: %(default)s",
     )
+    parser.add_argument(
+        "--cluster_name",
+        default=None,
+        help="Human friendly tron cluster name",
+    )
     parser.add_argument(
         "-s",
         "--save",
@@ -183,14 +193,28 @@ def save_config(options):
 
 def setup_logging(options):
     if options.verbose is None:
-        level = logging.WARNING
+        level = logging.CRITICAL
     elif options.verbose == 1:
+        level = logging.WARNING
+    elif options.verbose == 2:
         level = logging.INFO
     else:
-        level = logging.DEBUG
+        level = logging.NOTSET
 
     logging.basicConfig(
         level=level,
         format='%(name)s %(levelname)s %(message)s',
         stream=sys.stdout,
     )
+
+
+def suggest_possibilities(word, possibilities, max_suggestions=6):
+    suggestions = difflib.get_close_matches(
+        word=word, possibilities=possibilities, n=max_suggestions
+    )
+    if len(suggestions) == 1:
+        return f"\nDid you mean: {suggestions[0]}?"
+    elif len(suggestions) >= 1:
+        return f"\nDid you mean one of: {', '.join(suggestions)}?"
+    else:
+        return ""
