@@ -94,8 +94,9 @@ class TableDisplay(object):
 
     header_color = 'hgray'
 
-    def __init__(self):
+    def __init__(self, sort_index=0):
         self.out = []
+        self.sort_index = sort_index
 
     def banner(self):
         if not self.title:
@@ -155,7 +156,7 @@ class TableDisplay(object):
     def rows(self):
         return sorted(
             self.data,
-            key=itemgetter(self.fields[0]),
+            key=itemgetter(self.fields[self.sort_index]),
             reverse=self.reversed,
         )
 
@@ -197,11 +198,11 @@ class TableDisplay(object):
 
 
 def add_color_for_state(state):
-    if state == actionrun.ActionRun.STATE_FAILED.name:
+    if state == actionrun.ActionRun.FAILED:
         return Color.set('red', state)
     if state in {
-        actionrun.ActionRun.STATE_RUNNING.name,
-        actionrun.ActionRun.STATE_SUCCEEDED.name,
+        actionrun.ActionRun.RUNNING,
+        actionrun.ActionRun.SUCCEEDED,
         job.Job.STATUS_ENABLED,
     }:
         return Color.set('green', state)
@@ -219,10 +220,8 @@ def format_fields(display_obj, content):
         return display_obj.colors[field](field_value)
 
     def format_field(field):
-        value = content.get(field)
-        if value is None:
-            return ''
-        return field_display_mapping.get(field, lambda f: f)(value)
+        formatter = field_display_mapping.get(field, lambda f, _: f)
+        return formatter(content.get(field), content)
 
     def build_field(label, field):
         return "%-20s: %s" % (label, add_color(field, format_field(field)))
@@ -346,7 +345,7 @@ class DisplayActionRuns(TableDisplay):
 
     detail_labels = [
         ('Action Run', 'id'),
-        ('State', 'state'),
+        ('State', 'state_delayed'),
         ('Node', 'node'),
         ('Command', 'command'),
         ('Bare command', 'raw_command'),
@@ -354,6 +353,8 @@ class DisplayActionRuns(TableDisplay):
         ('End time', 'end_time'),
         ('Final exit status', 'exit_status'),
         ('Retry exit statuses', 'exit_statuses'),
+        ('Waits for triggers', 'triggered_by'),
+        ('Publishes triggers', 'trigger_downstreams'),
     ]
 
     colors = {
@@ -362,6 +363,10 @@ class DisplayActionRuns(TableDisplay):
         'command': partial(Color.set, 'gray'),
     }
 
+    def __init__(self):
+        # Action runs need to be storted by start time, which is index 2
+        super().__init__(sort_index=2)
+
     def banner(self):
         self.out.append(format_fields(DisplayJobRuns, self.job_run))
         super(DisplayActionRuns, self).banner()
@@ -384,32 +389,50 @@ class DisplayActionRuns(TableDisplay):
         self.data = data['runs']
         self.job_run = data
 
-
-class DisplayEvents(TableDisplay):
-
-    columns = ['Time', 'Level', 'Entity', 'Name']
-    fields = ['time', 'level', 'entity', 'name']
-    widths = [22, 12, 35, 20]
-    title = 'events'
-    resize_fields = ['entity']
+    def rows(self):
+        # Action runs need a sort order that sorts by date
+        # and that can handle situations where it is None, or
+        # othere weird things, so we str()
+        return sorted(
+            self.data,
+            key=lambda x: str(x[self.fields[self.sort_index]]),
+            reverse=self.reversed,
+        )
 
 
-def display_node(source):
+def display_node(source, _=None):
+    if not source:
+        return ''
     return '%s@%s' % (source['username'], source['hostname'])
 
 
-def display_node_pool(source):
+def display_node_pool(source, _=None):
+    if not source:
+        return ''
     return "%s (%d node(s))" % (source['name'], len(source['nodes']))
 
 
-def display_scheduler(source):
+def display_scheduler(source, _=None):
+    if not source:
+        return ''
     return "%s %s%s" % (source['type'], source['value'], source['jitter'])
 
 
+def display_state_delayed(_, obj):
+    state = obj['state']
+    in_delay = obj['in_delay']
+    if in_delay:
+        return f"{state} (retry delayed for {int(in_delay)}s)"
+    else:
+        return state
+
+
 field_display_mapping = {
     'node': display_node,
     'node_pool': display_node_pool,
     'scheduler': display_scheduler,
+    'state_delayed': display_state_delayed,
+    'exit_status': lambda v, _: actionrun.ActionRun.EXIT_REASONS.get(v, v),
 }
 
 
