@@ -5,8 +5,7 @@ from __future__ import absolute_import
 from __future__ import unicode_literals
 
 from collections import namedtuple
-
-from tron.utils.collections import Enum
+from enum import Enum
 
 MASTER_NAMESPACE = "MASTER"
 
@@ -42,29 +41,21 @@ TronConfig = config_object_factory(
         'output_stream_dir',  # str
         'action_runner',  # ConfigActionRunner
         'state_persistence',  # ConfigState
-        'command_context',  # FrozenDict of str
+        'command_context',  # dict of str
         'ssh_options',  # ConfigSSHOptions
-        'notification_options',  # NotificationOptions or None
         'time_zone',  # pytz time zone
-        'nodes',  # FrozenDict of ConfigNode
-        'node_pools',  # FrozenDict of ConfigNodePool
-        'jobs',  # FrozenDict of ConfigJob
-        'clusters',  # tuple of str
+        'nodes',  # dict of ConfigNode
+        'node_pools',  # dict of ConfigNodePool
+        'jobs',  # dict of ConfigJob
+        'mesos_options',  # ConfigMesos
+        'eventbus_enabled',  # bool or None
     ],
 )
 
 NamedTronConfig = config_object_factory(
     name='NamedTronConfig',
     optional=[
-        'jobs',  # FrozenDict of ConfigJob
-    ],
-)
-
-NotificationOptions = config_object_factory(
-    name='NotificationOptions',
-    required=[
-        'smtp_host',  # str
-        'notification_addr',  # str
+        'jobs',  # dict of ConfigJob
     ],
 )
 
@@ -102,8 +93,24 @@ ConfigState = config_object_factory(
         'store_type',
     ],
     optional=[
-        'connection_details',
         'buffer_size',
+        'dynamodb_region',
+        'table_name',
+    ],
+)
+
+ConfigMesos = config_object_factory(
+    name='ConfigMesos',
+    optional=[
+        'master_address',
+        'master_port',
+        'secret_file',
+        'principal',
+        'role',
+        'enabled',
+        'default_volumes',
+        'dockercfg_location',
+        'offer_timeout',
     ],
 )
 
@@ -113,7 +120,7 @@ ConfigJob = config_object_factory(
         'name',  # str
         'node',  # str
         'schedule',  # Config*Scheduler
-        'actions',  # FrozenDict of ConfigAction
+        'actions',  # dict of ConfigAction
         'namespace',  # str
     ],
     optional=[
@@ -126,8 +133,6 @@ ConfigJob = config_object_factory(
         'allow_overlap',  # bool
         'max_runtime',  # datetime.Timedelta
         'time_zone',  # pytz time zone
-        'service',  # str
-        'deploy_group',  # str
         'expected_runtime',  # datetime.Timedelta
     ],
 )
@@ -141,15 +146,22 @@ ConfigAction = config_object_factory(
     optional=[
         'requires',  # tuple of str
         'node',  # str
+        'retries',  # int
+        'retries_delay',  # datetime.Timedelta
         'executor',  # str
-        'cluster',  # str
-        'pool',  # str
         'cpus',  # float
         'mem',  # float
-        'service',  # str
-        'deploy_group',  # str
-        'retries',  # int
+        'disk',  # float
+        'constraints',  # List of ConfigConstraint
+        'docker_image',  # str
+        'docker_parameters',  # List of ConfigParameter
+        'env',  # dict
+        'extra_volumes',  # List of ConfigVolume
         'expected_runtime',  # datetime.Timedelta
+        'trigger_downstreams',  # None, bool or dict
+        'triggered_by',  # list or None
+        'on_upstream_rerun',  # ActionOnRerun or None
+        'trigger_timeout',  # datetime.deltatime or None
     ],
 )
 
@@ -161,20 +173,63 @@ ConfigCleanupAction = config_object_factory(
     optional=[
         'name',  # str
         'node',  # str
+        'retries',  # int
+        'retries_delay',  # datetime.Timedelta
+        'expected_runtime',  # datetime.Timedelta
         'executor',  # str
-        'cluster',  # str
-        'pool',  # str
         'cpus',  # float
         'mem',  # float
-        'service',  # str
-        'deploy_group',  # str
-        'retries',  # int
-        'expected_runtime',  # datetime.Timedelta
+        'disk',  # float
+        'constraints',  # List of ConfigConstraint
+        'docker_image',  # str
+        'docker_parameters',  # List of ConfigParameter
+        'env',  # dict
+        'extra_volumes',  # List of ConfigVolume
+        'trigger_downstreams',  # None, bool or dict
+        'triggered_by',  # list or None
+        'on_upstream_rerun',  # ActionOnRerun or None
+        'trigger_timeout',  # datetime.deltatime or None
     ],
 )
 
-StatePersistenceTypes = Enum.create('shelve', 'sql', 'yaml')
+ConfigConstraint = config_object_factory(
+    name='ConfigConstraint',
+    required=[
+        'attribute',
+        'operator',
+        'value',
+    ],
+    optional=[],
+)
 
-ExecutorTypes = Enum.create('ssh', 'paasta')
+ConfigVolume = config_object_factory(
+    name='ConfigVolume',
+    required=[
+        'container_path',
+        'host_path',
+        'mode',
+    ],
+    optional=[],
+)
 
-ActionRunnerTypes = Enum.create('none', 'subprocess')
+ConfigParameter = config_object_factory(
+    name='ConfigParameter',
+    required=[
+        'key',
+        'value',
+    ],
+    optional=[],
+)
+
+
+StatePersistenceTypes = Enum(
+    'StatePersistenceTypes', dict(shelve='shelve', yaml='yaml', dynamodb='dynamodb')
+)
+
+ExecutorTypes = Enum('ExecutorTypes', dict(ssh='ssh', mesos='mesos'))
+
+ActionRunnerTypes = Enum('ActionRunnerTypes', dict(none='none', subprocess='subprocess'))
+
+VolumeModes = Enum('VolumeModes', dict(RO='RO', RW='RW'))
+
+ActionOnRerun = Enum('ActionOnRerun', dict(rerun='rerun'))
