@@ -1,27 +1,26 @@
 """
  Classes to manage job runs.
 """
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
+import json
 import logging
+import time
 from collections import deque
 
-from six.moves import filter
-
+import tron.metrics as metrics
 from tron import command_context
-from tron import event
 from tron import node
 from tron.core.actionrun import ActionRun
 from tron.core.actionrun import ActionRunFactory
 from tron.serialize import filehandler
 from tron.utils import maybe_decode
+from tron.utils import next_or_none
 from tron.utils import proxy
 from tron.utils import timeutils
 from tron.utils.observer import Observable
 from tron.utils.observer import Observer
 
 log = logging.getLogger(__name__)
+state_logger = logging.getLogger(f'{__name__}.state_changes')
 
 
 class Error(Exception):
@@ -51,8 +50,6 @@ class JobRun(Observable, Observer):
         action_runs=None,
         action_graph=None,
         manual=None,
-        service=None,
-        deploy_group=None,
     ):
         super(JobRun, self).__init__()
         self.job_name = maybe_decode(job_name)
@@ -60,15 +57,11 @@ class JobRun(Observable, Observer):
         self.run_time = run_time
         self.node = node
         self.output_path = output_path or filehandler.OutputPath()
-        self.output_path.append(self.id)
+        self.output_path.append(str(self.run_num))
         self.action_runs_proxy = None
         self._action_runs = None
         self.action_graph = action_graph
         self.manual = manual
-        self.service = service
-        self.deploy_group = deploy_group
-        self.event = event.get_recorder(self.full_id)
-        self.event.ok('created')
 
         if action_runs:
             self.action_runs = action_runs
@@ -87,14 +80,11 @@ class JobRun(Observable, Observer):
             run_num,
             run_time,
             node,
-            job.output_path.clone(),
-            job.context,
+            output_path=job.output_path.clone(),
+            base_context=job.context,
             action_graph=job.action_graph,
             manual=manual,
-            service=job.service,
-            deploy_group=job.deploy_group,
         )
-
         action_runs = ActionRunFactory.build_action_run_collection(
             run,
             job.action_runner,
@@ -125,8 +115,6 @@ class JobRun(Observable, Observer):
             manual=state_data.get('manual', False),
             output_path=output_path,
             base_context=context,
-            service=state_data.get('service'),
-            deploy_group=state_data.get('deploy_group'),
         )
         action_runs = ActionRunFactory.action_run_collection_from_state(
             job_run,
@@ -147,8 +135,6 @@ class JobRun(Observable, Observer):
             'runs': self.action_runs.state_data,
             'cleanup_run': self.action_runs.cleanup_action_state_data,
             'manual': self.manual,
-            'service': self.service,
-            'deploy_group': self.deploy_group,
         }
 
     def _get_action_runs(self):
@@ -162,31 +148,23 @@ class JobRun(Observable, Observer):
         self._action_runs = run_collection
         for action_run in run_collection.action_runs_with_cleanup:
             self.watch(action_run)
+            action_run.setup_subscriptions()
 
         self.action_runs_proxy = proxy.AttributeProxy(
-            self.action_runs,
+            run_collection,
             [
                 'queue',
                 'cancel',
                 'success',
                 'fail',
-                'is_cancelled',
-                'is_unknown',
-                'is_failed',
-                'is_succeeded',
-                'is_running',
-                'is_starting',
-                'is_queued',
-                'is_scheduled',
-                'is_skipped',
-                'is_starting',
                 'start_time',
                 'end_time',
             ],
         )
 
     def _del_action_runs(self):
-        del self._action_runs
+        self._action_runs = None
+        self.action_runs_proxy = None
 
     action_runs = property(
         _get_action_runs,
@@ -203,17 +181,15 @@ class JobRun(Observable, Observer):
         return max(0, timeutils.delta_total_seconds(run_time - now))
 
     def start(self):
-        """Start this JobRun as a scheduled run (not a manual run)."""
-        self.event.info('start')
-        if self.action_runs.has_startable_action_runs and self._do_start():
+        self.log_state_update(state='start')
+        if self._do_start():
             return True
 
     def _do_start(self):
-        log.info("Starting JobRun %s", self.id)
-
+        log.info(f"{self} starting")
         self.action_runs.ready()
         if any(self._start_action_runs()):
-            self.event.ok('started')
+            log.info(f'{self} started')
             return True
 
     def stop(self):
@@ -225,17 +201,43 @@ class JobRun(Observable, Observer):
         """Start all startable action runs, and return any that were
         successfully started.
         """
-        started_actions = []
-        for action_run in self.action_runs.get_startable_action_runs():
-            if action_run.start():
-                started_actions.append(action_run)
+        started_runs = [
+            action_run
+            for action_run in self.action_runs.get_startable_action_runs()
+            if action_run.start()
+        ]
 
-        return started_actions
+        if not started_runs:
+            for r in self.action_runs:
+                if r.is_blocked_on_trigger:
+                    log.debug(f"{r} is blocked on triggers: {r.remaining_triggers}")
 
-    def handle_action_run_state_change(self, action_run, _):
+        return started_runs
+
+    def handle_action_run_state_change(self, action_run: ActionRun, event):
         """Handle events triggered by JobRuns."""
+        log.info(f"{self} got an event: {event}")
+        metrics.meter(f'tron.actionrun.{event}')
+
+        if event == ActionRun.NOTIFY_TRIGGER_READY:
+            if self.is_scheduled or self.is_queued:
+                log.info(f"{self} triggers are satisfied but run not started yet")
+                return
+
+            started = self._start_action_runs()
+            if any(started):
+                log.info(
+                    f"{self} action runs triggered: "
+                    f"{', '.join(str(s) for s in started)}"
+                )
+            return
+
         # propagate all state changes (from action runs) up to state serializer
         self.notify(self.NOTIFY_STATE_CHANGED)
+        self.log_state_update(
+            state=action_run.state,
+            action_name=action_run.name,
+        )
 
         if not action_run.is_done:
             return
@@ -243,25 +245,25 @@ class JobRun(Observable, Observer):
         if action_run.is_skipped and self.action_runs.is_scheduled:
             return
 
-        if not action_run.is_broken and any(self._start_action_runs()):
-            log.info("Action runs started for %s." % self)
-            return
+        if not action_run.is_broken:
+            started = self._start_action_runs()
+            if any(started):
+                log.info(
+                    f"{self} action runs started: "
+                    f"{', '.join(str(s) for s in started)}"
+                )
+                return
 
-        if self.action_runs.is_active or self.action_runs.is_scheduled:
-            log.info("%s still has running or scheduled actions." % self)
+        if not self.action_runs.is_done:
+            log.info(f"{self} still has running or waiting actions")
             return
 
         # If we can't make any progress, we're done
-        cleanup_run = self.action_runs.cleanup_action_run
+        cleanup_run: ActionRun = self.action_runs.cleanup_action_run
         if not cleanup_run or cleanup_run.is_done:
             return self.finalize()
 
-        # TODO: remove in (0.6), start() no longer raises an exception
-        # When a job is being disabled, or the daemon is being shut down a bunch
-        # of ActionRuns will be cancelled/failed. This would cause cleanup
-        # action to be triggered more then once. Guard against that.
-        if cleanup_run.check_state('start'):
-            cleanup_run.start()
+        cleanup_run.start()
 
     handler = handle_action_run_state_change
 
@@ -273,63 +275,85 @@ class JobRun(Observable, Observer):
         Triggers an event to notifies the Job that is is done.
         """
         if self.action_runs.is_failed:
-            self.event.critical('failed')
+            log.error(f'{self} failed')
         else:
-            self.event.ok('succeeded')
+            log.info(f'{self} succeeded')
 
         # Notify Job that this JobRun is complete
         self.notify(self.NOTIFY_DONE)
+        self.log_state_update(state=self.state)
 
     def cleanup(self):
         """Cleanup any resources used by this JobRun."""
-        self.event.notice('removed')
-        event.EventManager.get_instance().remove(self.full_id)
+        log.info(f'{self} removed')
         self.clear_observers()
         self.action_runs.cleanup()
         self.node = None
         self.action_graph = None
-        self._action_runs = None
+        self._del_action_runs()
         self.output_path.delete()
 
     def get_action_run(self, action_name):
         return self.action_runs.get(action_name)
 
+    def log_state_update(self, state, action_name=None):
+        if action_name is None:
+            state = f'job_{state}'
+        else:
+            action_name = str(action_name)
+
+        data = {
+            'job_name': str(self.job_name),
+            'run_num': str(self.run_num),
+            'action_name': action_name,
+            'state': str(state),
+            'timestamp': time.time(),
+        }
+        state_logger.info(json.dumps(data))
+
     @property
     def state(self):
         """The overall state of this job run. Based on the state of its actions.
         """
         if not self.action_runs:
-            log.info("%s has no state" % self)
-            return ActionRun.STATE_UNKNOWN
+            log.info(f"{self} has no action runs to determine state")
+            return ActionRun.UNKNOWN
 
         if self.action_runs.is_complete:
-            return ActionRun.STATE_SUCCEEDED
+            return ActionRun.SUCCEEDED
         if self.action_runs.is_cancelled:
-            return ActionRun.STATE_CANCELLED
+            return ActionRun.CANCELLED
         if self.action_runs.is_running:
-            return ActionRun.STATE_RUNNING
+            return ActionRun.RUNNING
         if self.action_runs.is_starting:
-            return ActionRun.STATE_STARTING
+            return ActionRun.STARTING
         if self.action_runs.is_failed:
-            return ActionRun.STATE_FAILED
+            return ActionRun.FAILED
+        if self.action_runs.is_waiting and self.action_runs.is_blocked_on_trigger:
+            return ActionRun.WAITING
         if self.action_runs.is_scheduled:
-            return ActionRun.STATE_SCHEDULED
+            return ActionRun.SCHEDULED
         if self.action_runs.is_queued:
-            return ActionRun.STATE_QUEUED
+            return ActionRun.QUEUED
 
-        return ActionRun.STATE_UNKNOWN
+        return ActionRun.UNKNOWN
 
-    @property
-    def full_id(self):
-        return "JobRun:%s" % self.id
+    def cancel(self):
+        return self.action_runs.cancel()
 
     def __getattr__(self, name):
-        if self.action_runs_proxy:
+        if name.startswith('is_'):
+            state_name = name[3:]
+            if state_name not in ActionRun.STATE_MACHINE.states:
+                raise RuntimeError(f"{state_name} not in ActionRun.VALID_STATES")
+            return self.state == state_name
+        elif self.action_runs_proxy:
             return self.action_runs_proxy.perform(name)
-        raise AttributeError(name)
+        else:
+            raise AttributeError(name)
 
     def __str__(self):
-        return self.full_id
+        return f"JobRun:{self.id}"
 
 
 class JobRunCollection(object):
@@ -358,16 +382,8 @@ class JobRunCollection(object):
         and return it.
         """
         run_num = self.next_run_num()
-        log.info(
-            "Building JobRun %s for %s on %s at %s" % (
-                run_num,
-                job,
-                node,
-                run_time,
-            )
-        )
-
         run = JobRun.for_job(job, run_num, run_time, node, manual)
+        log.info(f"{run} created on {node.name} at {run_time}")
         self.runs.appendleft(run)
         self.remove_old_runs()
         return run
@@ -383,31 +399,13 @@ class JobRunCollection(object):
             pending.cleanup()
             self.runs.remove(pending)
 
-    def _get_runs_using(self, func, reverse=False):
-        """Filter runs using func()."""
-        job_runs = self.runs if not reverse else reversed(self.runs)
-        return filter(func, job_runs)
-
-    def _get_run_using(self, func, reverse=False):
-        """Find the first run (from most recent to least recent), where func()
-        returns true.  func() should be a callable which takes a single
-        argument (a JobRun), and return True or False.
-        """
-        try:
-            return next(self._get_runs_using(func, reverse))
-        except StopIteration:
-            return None
-
-    def _filter_by_state(self, state):
-        return lambda r: r.state == state
-
     def get_run_by_state(self, state):
         """Returns the most recent run which matches the state."""
-        return self._get_run_using(self._filter_by_state(state))
+        return next_or_none(r for r in self.runs if r.state == state)
 
     def get_run_by_num(self, num):
         """Return a the run with run number which matches num."""
-        return self._get_run_using(lambda r: r.run_num == num)
+        return next_or_none(r for r in self.runs if r.run_num == num)
 
     def get_run_by_index(self, index):
         """Return the job run at index. Jobs are indexed from oldest to newest.
@@ -417,64 +415,35 @@ class JobRunCollection(object):
         except IndexError:
             return None
 
-    def get_run_by_state_short_name(self, short_name):
-        """Returns the most recent run which matches the state short name."""
-        return self._get_run_using(lambda r: r.state.short_name == short_name)
-
     def get_newest(self, include_manual=True):
         """Returns the most recently created JobRun."""
-
-        def func(r):
-            return True if include_manual else not r.manual
-
-        return self._get_run_using(func)
+        return next_or_none(
+            r for r in self.runs if include_manual or not r.manual
+        )
 
     def get_pending(self):
         """Return the job runs that are queued or scheduled."""
-        return self._get_runs_using(lambda r: r.is_scheduled or r.is_queued)
+        return [r for r in self.runs if r.state in (ActionRun.SCHEDULED, ActionRun.QUEUED)]
 
     @property
     def has_pending(self):
         return any(self.get_pending())
 
     def get_active(self, node=None):
-        if node:
-
-            def func(r):
-                return (r.is_running or r.is_starting) and r.node == node
-        else:
-
-            def func(r):
-                return r.is_running or r.is_starting
-
-        return self._get_runs_using(func)
+        active_states = {ActionRun.RUNNING, ActionRun.STARTING, ActionRun.WAITING}
+        return [
+            r for r in self.runs if (r.state in active_states) and
+            (not node or r.node == node)
+        ]
 
     def get_first_queued(self, node=None):
-        state = ActionRun.STATE_QUEUED
-        if node:
-
-            def queued_func(r):
-                return r.state == state and r.node == node
-        else:
-            queued_func = self._filter_by_state(state)
-        return self._get_run_using(queued_func, reverse=True)
+        return next_or_none(
+            r for r in reversed(self.runs)
+            if (not node or r.node == node) and r.state == ActionRun.QUEUED
+        )
 
     def get_scheduled(self):
-        state = ActionRun.STATE_SCHEDULED
-        return self._get_runs_using(self._filter_by_state(state))
-
-    def get_next_to_finish(self, node=None):
-        """Return the most recent run which is either running or scheduled. If
-        node is not None, then only looks for runs on that node.
-        """
-
-        def compare(run):
-            if node and run.node != node:
-                return False
-            if run.is_running or run.is_scheduled:
-                return run
-
-        return self._get_run_using(compare)
+        return [r for r in self.runs if r.state == ActionRun.SCHEDULED]
 
     def next_run_num(self):
         """Return the next run number to use."""
@@ -491,7 +460,7 @@ class JobRunCollection(object):
             run.cleanup()
 
     def get_action_runs(self, action_name):
-        return [job_run.get_action_run(action_name) for job_run in self]
+        return [job_run.get_action_run(action_name) for job_run in self.runs]
 
     @property
     def state_data(self):
@@ -500,11 +469,11 @@ class JobRunCollection(object):
 
     @property
     def last_success(self):
-        return self.get_run_by_state(ActionRun.STATE_SUCCEEDED)
+        return self.get_run_by_state(ActionRun.SUCCEEDED)
 
     @property
     def next_run(self):
-        return self.get_run_by_state(ActionRun.STATE_SCHEDULED)
+        return self.get_run_by_state(ActionRun.SCHEDULED)
 
     def __iter__(self):
         return iter(self.runs)
