@@ -1,11 +1,7 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import itertools
 import logging
 import random
 
-import six
 from twisted.conch.client.knownhosts import KnownHostsFile
 from twisted.internet import defer
 from twisted.internet import protocol
@@ -13,7 +9,6 @@ from twisted.internet import reactor
 from twisted.python import failure
 from twisted.python.filepath import FilePath
 
-from tron import eventloop
 from tron import ssh
 from tron.utils import collections
 from tron.utils import twistedutils
@@ -102,13 +97,13 @@ class NodePoolRepository(object):
         known_hosts,
         ssh_config,
     ):
-        for config in six.itervalues(node_configs):
+        for config in node_configs.values():
             pub_key = known_hosts.get_public_key(config.hostname)
             node = Node.from_config(config, ssh_options, pub_key, ssh_config)
             self.add_node(node)
 
     def _update_node_pools(self, node_pool_configs):
-        for config in six.itervalues(node_pool_configs):
+        for config in node_pool_configs.values():
             nodes = self._get_nodes_by_name(config.nodes)
             pool = NodePool.from_config(config, nodes)
             self.pools.replace(pool)
@@ -245,7 +240,7 @@ class Node(object):
         # Map of run id to instance of RunState
         self.run_states = {}
 
-        self.idle_timer = eventloop.NullCallback
+        self.idle_timer = None
         self.disabled = False
         self.pub_key = pub_key
 
@@ -278,10 +273,10 @@ class Node(object):
         if not isinstance(other, self.__class__):
             return False
         return (
-            self.config == other.config
-            and self.conch_options == other.conch_options
-            and self.pub_key == other.pub_key
-            and self.node_settings == other.node_settings
+            self.config == other.config and
+            self.conch_options == other.conch_options and
+            self.pub_key == other.pub_key and
+            self.node_settings == other.node_settings
         )
 
     def __ne__(self, other):
@@ -305,7 +300,7 @@ class Node(object):
         log.info("Running %s for %s on %s", run.command, run.id, self.hostname)
 
         # When this run completes, for good or bad, we'll inform the caller by
-        # calling 'succeed' or 'fail' on the run Since the definined interface
+        # calling 'succeed' or 'fail' on the run Since the defined interface
         # is on these specific callbacks, we won't bother returning the
         # deferred here. This allows the caller to not really care about
         # twisted specific stuff at all, all it needs to know is that one of
@@ -318,7 +313,7 @@ class Node(object):
                 self.run_states[run.id],
             )
 
-        if self.idle_timer.active():
+        if self.idle_timer and self.idle_timer.active():
             self.idle_timer.cancel()
 
         self.run_states[run.id] = RunState(run)
@@ -336,7 +331,7 @@ class Node(object):
                 run.id,
                 fudge_factor,
             )
-            eventloop.call_later(fudge_factor, self._do_run, run)
+            reactor.callLater(fudge_factor, self._do_run, run)
 
         # We return the deferred here, but really we're trying to keep the rest
         # of the world from getting too involved with twisted.
@@ -366,7 +361,7 @@ class Node(object):
         del self.run_states[run.id]
 
         if not self.run_states:
-            self.idle_timer = eventloop.call_later(
+            self.idle_timer = reactor.callLater(
                 self.node_settings.idle_connection_timeout,
                 self._connection_idle_timeout,
             )
@@ -388,7 +383,7 @@ class Node(object):
             return
 
         # Add a dummy errback handler to prevent Unhandled error messages.
-        # Unless somone is explicitly caring about this defer the error will
+        # Unless someone is explicitly caring about this defer the error will
         # have been reported elsewhere.
         self.run_states[run.id].deferred.addErrback(lambda failure: None)
 
@@ -414,9 +409,10 @@ class Node(object):
 
         def connect_fail(result):
             log.warning(
-                "Cannot run %s, Failed to connect to %s",
+                "Cannot run %s, Failed to connect to %s: %s",
                 run,
                 self.hostname,
+                repr(result)
             )
             self.connection_defer = None
             self._fail_run(
@@ -446,9 +442,9 @@ class Node(object):
             return
         self.connection = None
 
-        log.info("Service to %s stopped", self.hostname)
+        log.info(f"Service to node {self.hostname} stopped")
 
-        for run_id, run in six.iteritems(self.run_states):
+        for run_id, run in self.run_states.items():
             if run.state == RUN_STATE_CONNECTING:
                 # Now we can trigger a reconnect and re-start any waiting runs.
                 self._connect_then_run(run)
@@ -459,7 +455,7 @@ class Node(object):
 
                     # This means our run IS still waiting to start. There
                     # should be an outstanding timeout sitting on this guy as
-                    # well. We'll just short circut it.
+                    # well. We'll just short circuit it.
                     twistedutils.defer_timeout(run.channel.start_defer, 0)
                 else:
                     # Doesn't seem like this should ever happen.
@@ -496,6 +492,7 @@ class Node(object):
         create_defer = client_creator.connectTCP(
             self.hostname,
             self.config.port,
+            timeout=self.node_settings.connect_timeout,
         )
 
         # We're going to create a deferred, returned to the caller, that will
@@ -602,7 +599,7 @@ class Node(object):
 
         We don't actually know if the run succeeded
         """
-        log.error("Failure waiting on channel completion: %s", str(result))
+        log.error("Failure waiting on channel completion: %s", repr(result))
         self._fail_run(run, failure.Failure(exc_value=ResultError()))
 
     def _run_started(self, channel, run):
@@ -612,9 +609,6 @@ class Node(object):
         if not self._is_run_id_tracked(run):
             log.warning("Run %s no longer tracked (_run_started)", run.id)
             return
-        assert self.run_states[run.id].state == RUN_STATE_STARTING
-        self.run_states[run.id].state = RUN_STATE_RUNNING
-
         run.started()
 
     def _run_start_error(self, result, run):
@@ -626,7 +620,7 @@ class Node(object):
             "Error running %s, disconnecting from %s: %s",
             run.id,
             self.hostname,
-            str(result),
+            repr(result),
         )
 
         # We clear out the deferred that likely called us because there are
