@@ -46,8 +46,8 @@ class SSHAuthOptions(object):
 
     def __eq__(self, other):
         return other and (
-            self.use_agent == other.use_agent
-            and self.identitys == other.identitys
+            self.use_agent == other.use_agent and
+            self.identitys == other.identitys
         )
 
     def __ne__(self, other):
@@ -60,9 +60,18 @@ class SSHAuthOptions(object):
 
 class NoPasswordAuthClient(default.SSHUserAuthClient):
     """Only support passwordless auth."""
-    preferredOrder = ['publickey']
+    preferredOrder = ['publickey', 'keyboard-interactive']
     auth_password = None
-    auth_keyboard_interactive = None
+
+    def getGenericAnswers(self, name, instruction, prompts):
+        # We really only need to get input from the user if there is actually a prompt
+        # This works around an issue where some PAM modules that have "keyboard-interactive"
+        # but don't actually require any input from the end-user
+        if prompts:
+            return super().getGenericAnswers(self, name, instruction, prompts)
+
+        # Otherwise, just return an empty defer.succeed to satisfy the contract
+        return defer.succeed([])
 
 
 class ClientTransport(transport.SSHClientTransport):
@@ -104,15 +113,28 @@ class ClientConnection(connection.SSHConnection):
     service_start_defer = None
     service_stop_defer = None
 
+    def setToFromStrings(self):
+        try:
+            self.from_string = f"{self.transport.getHost().address.host}:{self.transport.getHost().address.port}"
+        except Exception as e:
+            log.debug(e)
+            self.to_string = "(unknown tcp source)"
+        try:
+            self.to_string = f"{self.transport.getPeer().address.host}:{self.transport.getPeer().address.port}"
+        except Exception as e:
+            log.debug(e)
+            self.to_string = "(unknown tcp destination)"
+
     def serviceStarted(self):
-        log.info("Service started")
         connection.SSHConnection.serviceStarted(self)
+        self.setToFromStrings()
+        log.info(f"Started SSH connection from {self.from_string} to {self.to_string}")
         if not self.service_stop_defer.called:
             self.service_start_defer.callback(self)
 
     def serviceStopped(self):
-        log.info("Service stopped")
         connection.SSHConnection.serviceStopped(self)
+        log.info(f"Stopped SSH connection from {self.from_string} to {self.to_string}")
         if not self.service_stop_defer.called:
             self.service_stop_defer.callback(self)
 
@@ -232,8 +254,8 @@ class ExecChannel(channel.SSHChannel):
 
     def closed(self):
         if (
-            self.exit_status is None and self.running and self.exit_defer
-            and not self.exit_defer.called
+            self.exit_status is None and self.running and self.exit_defer and
+            not self.exit_defer.called
         ):
             log.warning(
                 "Channel has been closed without receiving an exit"
