@@ -1,6 +1,3 @@
-from __future__ import absolute_import
-from __future__ import unicode_literals
-
 import yaml
 
 
@@ -17,3 +14,8 @@ def load(*args, **kwargs):
 def load_all(*args, **kwargs):
     kwargs['Loader'] = yaml.CSafeLoader
     return yaml.load_all(*args, **kwargs)
+
+
+safe_dump = dump
+safe_load = load
+safe_load_all = load_all
